case $CONFIG in
'')
	if test ! -f config.sh; then
		ln ../config.sh . || \
		ln ../../config.sh . || \
		ln ../../../config.sh . || \
		(echo "Can't find config.sh."; exit 1)
	fi 2>/dev/null
	. config.sh
	;;
esac
case "$0" in
*/*) cd `expr X$0 : 'X\(.*\)/'` ;;
esac
echo "Extracting kit/mailkit (with variable substitutions)"
$spitshell >mailkit <<!GROK!THIS!
# @(#) Mailer for kit files
# @(#) (c) R. Manfredi, 1990 1991

# $Id: mailkit.SH,v 2.0.1.4 91/12/08 14:55:22 ram Exp $
#
# $Log:	mailkit.SH,v $
# Revision 2.0.1.4  91/12/08  14:55:22  ram
# patch10: forgot to protect evaluation of variable in a test
# 
# Revision 2.0.1.3  91/11/25  12:09:23  ram
# patch8: stamping of kit parts is now done directly in kit
# patch8: added option -l to allow taking recipients from a file
# 
# Revision 2.0.1.2  91/07/30  17:26:26  ram
# patch6: config.sh now carries the symbols defined in patchlevel.h
# patch6: the subject line in mails now appears first for USG /bin/mail
# 
# Revision 2.0.1.1  91/05/27  14:28:37  ram
# patch5: added option -n
# patch5: usage is no longer printed by a function
# patch5: version number is no longer hardwired
# patch5: set -- was not supported everywhere; use set X and shift
# 
# Revision 2.0  91/02/19  15:49:57  ram
# Baseline for first official release.
# 

# Define the mailer program
mailer="$mailer"

# Define the default editor
editor=$defeditor

# Which patch-level is it ?
pl='$PATCHLEVEL'
version='$VERSION'
!GROK!THIS!
$spitshell >>mailkit <<'!NO!SUBS!'

# Flag used for -e/f and -E/F options
edit=no
added=no
feed=no
preserve=no
recipients=''

# Default part's name
partname="Kit"

# Old Bourne shells do not have functions
usage='
echo "Usage: mailkit [-EFVefhp] [-n base] [-l file] prog_name [address_list]";
echo " "" -E : edit a message that will be sent as part #0";
echo " "" -F : feed message from stdin and sent it as part #0";
echo " "" -V : print version and patch-level";
echo " "" -e : edit a message that will be added to every part";
echo " "" -f : feed message from stdin to every part";
echo " "" -h : to print this help message.";
echo " "" -l : get recipients from a file (address_list is ignored).";
echo " "" -n : set the basename of parts to be sent (default is $partname).";
echo " "" -p : preserve files sent.";
exit 1;
'

# Argument parsing
for i in $*
do
	case $i in
	-V) echo "mailkit $version PL$pl"
		exit 0
		;;
	-E) edit=yes
		added=no
		shift
		;;
	-e) edit=yes
		added=yes
		shift
		;;
	-h) eval $usage
		;;
	-p) preserve=yes
		shift
		;;
	-l) recipients="$2"
		shift
		shift
		;;
	-n) partname="$2"
		shift
		shift
		;;
	-f)	feed=yes
		added=yes
		shift
		;;
	-F)	feed=yes
		added=no
		shift
		;;
	-*) echo "unknown option $1"
		eval $usage
		;;
	esac
done

if test "x$1" = x; then
	echo >&2 "$0: missing program name"
	eval $usage
fi

# Save arguments before using set
this_prog=$0
prog_name=$1
shift
address=$@
saved_address=$@

# When a recipient file is provided, ignore the remaining of the command line
if test "$recipients"; then
	if test -r "$recipients"; then
		# The format of the file is simple: a list of addresses, one per
		# line. Shell-style comments are allowed and multiple addresses on a
		# single line may be separated with spaces or commas.
		set X `<$recipients sed -n -e '/^[^#]/ {'	\
			-e 's/#.*//'	\
			-e 's/,/ /g'	\
			-e 's/\n/ /'	\
			-e 'p'			\
			-e '}'` $saved_address
		shift
		address=$@
	else
		echo >&2 "$this_prog: can't open $recipients"
		exit 1
	fi
fi

if test "x$address" = x; then
	echo >&2 "$this_prog: missing recipient list"
	eval $usage
fi

# In case of editing, compute a temporary file name
message=/tmp/MKt$$
# Temporary files used for building top message
tmpmsg=/tmp/Nkt$$
topmsg=/tmp/Tkt$$

# Set a trap in case we die abruptly
trap 'rm -f $message $tmpmsg $topmsg;
test $preserve = no && rm -f $partname??;
echo "Mailing aborted."
exit 1' 1 2 3 15

# Count how many files there are
set X $partname??
shift
nfiles=$#

if test $edit = yes
then
	# If editing was asked for, prepare the help message
	cat >$message <<EOM
Please, enter your message here for the "$prog_name" kit.
Do not worry about this text, the 4 lines at the top will be removed.
EOM
	if test $added = yes
	then
		cat >>$message <<'EOM'
The message will be added at the top of each part.
EOM
	else
		cat >>$message <<'EOM'
The message will be sent as part #0 and other parts will mention that.
EOM
	fi
	cat >>$message <<'EOM'
------------------------------------------------------------------------
EOM
	# Call the editor
	$editor $message

	# Now remove the 4 lines at the top
	# Do not use tail +5, as this may not exist on every system
	cp $message $tmpmsg
	awk '{ nl += 1 }; nl > 4 { print }' <$tmpmsg >$message
	rm -f $tmpmsg

	# If message is empty, discard it.
	if test ! -s $message
	then
		rm -f $message
		edit=no
	fi
elif test $feed = yes; then
	edit=yes			# No differences from now on
	cat > $message		# Feed from stdin
fi

# Print recipient list
echo "Mailing \"$prog_name\" to the following recipient list:"
echo $address | tr ' ' '\012' | sed -e 's/^/	/'

for i in $partname??
do
	# Extracts the part number (will it work on all systems ?)
	# Note that this does not rely on the leader part of the name.
	part=`expr $i : '[^0-9]*\(.*\)'`

	# Remove leading zero(s).
	# Is there a simpler way to do this ?
	part_num=`expr ${part:-1} + 0`

	# Protect against unexisting/unreadable files matching "$partname*"
	if  test -f $i -a -r $i
	then
		echo Mailing $i \($prog_name\)
		# System V /bin/mail does not recognize header correctly, unless the
		# first line is a subject. Oh well...
		cat > $topmsg <<EOH
Subject: $prog_name - kit #$part_num / $nfiles
To: $address

EOH
		# Has edit been requested ?
		if test $edit = yes
		then
			if test $added = yes
			then
				# Add message to each part
				cat $message >> $topmsg
				# add a blank line
				echo " " >> $topmsg
			else
				# Signals that there is a part #0
				cat >> $topmsg <<EOM
[ There is an instruction mail which is labeled: $prog_name - kit #0 / $nfiles ]

EOM
			fi
		fi
		cat $i >> $topmsg
		$mailer $address < $topmsg
		rm -f $topmsg
	else
		echo $this_prog: could not open $i >&2
	fi
done

# Is there a part #0 to send ?
if test $edit = yes -a $added = no
then
	cat > $topmsg <<EOM
Subject: $prog_name - kit #0 / $nfiles
To: $address

[ This is an instruction message for the $prog_name kit ]

EOM
	cat >>$topmsg <$message
	echo Mailing instructions \($prog_name\)
	$mailer $address < $topmsg
	rm -f $topmsg
fi

# Clean up
rm -f $message
test $preserve = no && rm -f $partname??
!NO!SUBS!
chmod 755 mailkit
$eunicefix mailkit
