.po -3
.nh
.TH "Collection" n "automatically Fri Mar  1 10:57:30 1991" "ET++ version v2.2" "ET++ class description"
.SH NAME
Collection.short \- base class for collections of objects
.SH DESCRIPTION
The abstract class \fBCollection\fP defines and partially implements the
protocol for managing a collection of objects. Objects put in a collection
must be instances of classes derived from the \fIclass Object\fP. These
classes should implement the following three methods:
.na
.nf

   \fB       IsEqual\fP        used in all collections for object
identification
   \fB       Compare\fP        if objects are put in sorted collections
(instances of \fIclass SortedObjList\fP)
   \fB       Hash\fP           if objects are put in collections based on hash
structures (instances of \fIclass Set\fP)

.ad b
.fi
\fBMemory Management\fP
.br

.br
The collection classes store only \fBreferences (pointers) to
objects\fP and not the objects themselves.  Due to the possibility that an
object is contained in more than one collection, a collection cannot decide
when an object should be deleted. For this reason all collections use the
policy to manage only the storage for the collection data structures but
not for the objects stored in the collection. The decision when to delete
an object is up to the client of a collection. Deleting a collection does
not delete the managed objects. The \fImethod FreeAll\fP can be used to
actually delete the collected objects. But take care that the deleted
objects are no longer referenced by other objects.
.br

.br
\fBCollection Types\fP
.br

.br
The subclasses of \fBCollection\fP implement different ways of storing and
accessing the objects. These different approaches result out of certain
trade-offs between needed functionality, flexibility and efficiency. The
inheritance relationship among the different classes is shown in the
following table:
.br
\fBCollection           \fP abstract superclass for all collection classes
.br

\fB  Set                \fP hash table
.br
\fB      IdSet          \fP hash table based on the address of an object
.br
\fB    Dictionary       \fP data structure for storing key value pairs
.br
\fB      IdDictionary   \fP same as above, but the address of an object 
is used as key
.br
\fB  ObjArray           \fP array of object pointers, with range checking 
and dynamic grow and shrink
.br
\fB  SeqCollection      \fP abstract superclass for collections
that preserve the order in which objects are added
.br
\fB    ObjList          \fP doubly linked list
.br
\fB      SortedObjList  \fP sorted list based on
\fBObject::Compare()\fP
.br
\fB      OrdCollection  \fP Ordered Collection - array based implementation 
of a list
.br

.br
\fBCollection\fP implements several methods to convert a collection to
another type: \fImethod AsBag AsSet AsObjArray AsObjList
AsOrderedCollection AsSortedObjList\fP. The resulting collection of a
conversion may contain less objects than the original collection, e.g. if a
\fBBag\fP is converted to a \fBSet\fP, multiple occurrences of an object
are lost.
.br

.br
\fBRetrieval of Elements\fP
.br

.br
\fBCollection\fP provides two different mechanisms for retrieval
of elements. The first involves a \fBpointer comparison\fP
between the search argument and the elements in the collection. This way of
comparing is often referred to as \fBidentity test\fP.
.br
The other way is based on the \fImethod @Object::IsEqual\fP of the
searched argument. This way of comparing is often referred to as \fBequality test\fP.
.br
This distinction is of central importance for collections because the \fImethod Object::IsEqual\fP does not uniquely identify an object. The
duality of object identification is also reflected in the corresponding
methods for finding and accessing objects. Examples are \fImethod
Find FindPtr\fP, \fImethod OccurrencesOf OccurrencesOfPtr\fP and \fImethod Remove RemovePtr\fP. Methods whose names end in \fB...Ptr\fP use
pointer comparison, the others are based on the \fImethod
Object::IsEqual\fP.
.br
The elements of a sequenceable collection can additionally be accessed by
the \fImethod At\fP. For example, \fBfoo-\>At(0)\fP returns the first
entry, \fBfoo-\>At(1)\fP the second, and so on. The last entry is numbered
one less than the value given by the \fImethod Size\fP.
.br

.br
\fBIterators\fP
.br

.br
Getting the objects of a collection one by one is done by a special object
called \fBiterator\fP. The order in which the objects are visited
depends on the type of the collection. Objects in a sequenceable collection
are visited in the order in which they have been added or sorted. Objects
of other collections are visited in an undefined order. The abstract
interface of all iterators is defined by the \fIclass Iterator\fP.
Every collection provides an own iterator which is returned by the \fImethod MakeIterator\fP (to reduce the memory overhead ET++ uses its own
memory management for iterators).
.br
Since iterators are typically short living objects, the \fIclass
Iter\fP provides an easy to use interface to them. This class takes care of
freeing a dynamically allocated iterator in its destructor (see example
below).
.br
The \fImethod Iter::operator()\fP returns the objects of the collection
one by one. Since the method returns an object of type \fBObject*\fP it
typically has to be casted to a more specific type before an operation can
be executed.
.br
For security reasons \fBCollection\fP implements a mechanism called
\fBRobust Iterators\fP. If an object is removed from a collection
while an iterator is active, then the object is replaced by an instance of
the \fIclass DeletedObject\fP. Iterators ignore "deleted objects".
These "place holders" are kept in the collection until the last iterator
has finished its work. See \fItechnote 'Robust Iterators'\fP for more
details.
.br
The following example shows how to use iterators:
.na
.nf
 
{

    OrdCollection *shapes;              
    VObject *current;
    Iter next (shapes);        // Create an iterator named 'next'

    while (current = (VObject*)next())   // Get objects one by one and cast them
    {
	current-\>Draw(rect);    // Use the 'current' VObject
    }

}   // End of block, the destructor of 'next' deletes the dynamically
allocated iterator

.ad b
.fi
A method can be applied to all objects of a collection with the \fImacro
ForEach\fP:
.na
.nf

    Collection *col;
    bool on;
    col-\>ForEach(VObject,Enable)(on,redraw);

.ad b
.fi
In this case \fBcol\fP is a collection of \fBVObjects\fP and the method \fBEnable\fP is called for all its elements with the arguments \fBon\fP and \fBredraw\fP.
.br

.br
\fBEnumerating Objects\fP
.br

.br
\fBCollection\fP implements two methods which are similar to the
operations selection and projection of the set theory.
.br
The \fImethod Select\fP creates a subset of a collection. The \fImethod Collect\fP creates a projection of a collection.

.br
classes are always derived from \fBCollection\fP.
.br
class \fBCollection\fP
is never reused directly.
.br
class \fBCollection\fP is abstract.
.br
class \fBCollection\fP contains 49 methods.
.PD 0
.TP 20
\fBowner of class:\fR
nobody.
.PD 0
.TP 20
\fBbaseclasses:\fR
.na
Object
.ad b
.PD 0
.TP 20
\fBsubclasses:\fR
.na
Bag, ObjArray, RunArray, SeqCollection, Set
.ad b
.PD 0
.TP 20
\fBfriend classes:\fR
.na
CollFilterIter
.ad b
.PD 0
.TP 20
\fBflags:\fR
ObjFlags

.SH INSTANCE VARIABLES
.TP
.BR iterCount " (private int)"
is the number of active iterators on this
collection. It is manipulated by \fImethod EnterIter\fP and \fImethod
ExitIter\fP which increment/decrement its value. The value is checked when objects
are to be added or removed.

.TP
.BR nDeleted " (private int)"
is the number of deleted objects in this collection. When a collection has
active iterators, removed objects are replaced with instances of \fIclass DeletedObject\fP. These dummy objects are actually removed when the
last iterator has finished its work.

.TP
.BR size " (protected int)"
is the number of objects contained in this collection (including deleted
objects). Subclasses increment or decrement this instance variable when
objects are added to, respectively removed from the collection. The maximum
number of elements in a collection is \fIconst cMaxInt\fP.


.SH INSTANCE METHOD LIST

.PD 0
.TP 20
accessing
At
.br
Find
.br
FindPtr
.br
Size

.PD 0
.TP 20
client interface
Contains
.br
ContainsPtr
.br
Hash
.br
OccurrencesOf
.br
OccurrencesOfPtr

.PD 0
.TP 20
conversion
AsBag
.br
AsObjArray
.br
AsObjList
.br
AsOrderedCollection
.br
AsSet
.br
AsSortedObjList

.PD 0
.TP 20
copying
Clone

.PD 0
.TP 20
debugging
DisplayOn

.PD 0
.TP 20
destruction
FreeAll

.PD 0
.TP 20
destructor
~Collection

.PD 0
.TP 20
enumeration
Collect
.br
Detect
.br
Select

.PD 0
.TP 20
fire walls
NullPointerWarning
.br
assertclass

.PD 0
.TP 20
implementation
AnnounceRemove
.br
AnyDeleted
.br
CheckActiveIter
.br
CheckNotNull
.br
EnterIter
.br
ExitIter
.br
GrowBy
.br
RemoveDeleted

.PD 0
.TP 20
internal
setCurrentCollection

.PD 0
.TP 20
iteration
InIterator
.br
MakeIterator

.PD 0
.TP 20
management
Collection

.PD 0
.TP 20
manipulation
Add
.br
AddAll
.br
AddVector
.br
Empty
.br
Remove
.br
RemoveAll
.br
RemovePtr

.PD 0
.TP 20
miscellaneous
InspectorId
.br
IsEqual
.br
PrintOn
.br
ReadFrom

.PD 0
.TP 20
testing
IsEmpty

.SH CATEGORIES

.na
Collection
.ad b

.SH FILES
.PD 0
.TP 20
declaration:
\fBCollection.h

.SH HISTORY
chris         Mar 20 1991 - first version
.br
gil           May 22 1991 - reworked
.br
chris         Jun  6 1991 - reworked
.br
joe           Jun 10 1991 - reviewed
.br
chris,joe     Jun 11 1991 - final version
.br
T. Kofler/SBG Sep  5 1991 - all method descriptions reworked
.br
