.po -3
.nh
.TH "CollectionView" n "automatically Mon Apr  8 20:18:15 1991" "ET++ version v2.2" "ET++ class description"
.SH NAME
CollectionView \- displays a list of selectable items in tabular format
.SH DESCRIPTION
The class \fBCollectionView\fP displays a list of \fBVObjects\fP in a tabular format similar to that of the \fIclass Cluster\fP and allows the selection of items. \fBCollectionView\fP implements the general behavior for all kinds of lists of selectable items such as popup and pulldown menus, menubars and tool palettes.
.br

.br
\fBItem Collection\fP
.br

.br
The list of items is stored in the \fIinstvar coll\fP. This collection is specified in the constructor and can additionally be set with the \fImethod SetCollection\fP. It is checked whether all items in the collection are instances of the \fIclass VObject\fP. If objects are added to the collection later, then the client itself has to ensure that they are "kind of" \fBVObject\fP.
.br
The \fBCollectionView\fP registers itself as an observer of the \fIinstvar coll\fP which enables it to recalculate the layout whenever the collection changes.
.br
Any type of \fBCollection\fP may be used. But if it is heavily modified and execution speed is important, then it is recommended to use an instance of the \fIclass OrdCollection\fP because \fBCollectionView\fP very often accesses items in the collection by index.
.br
The destructor of \fBCollectionView\fP always destroys the collection \fBand\fP the contained items. If the items are to be reused, then either single items or the whole collection should be saved before deleting the \fBCollectionView\fP.
.br

.br
\fBLayout\fP
.br

.br
The items of a \fBCollectionView\fP are arranged in \fBcolumns\fP and \fBrows\fP and the available space for an item is referred to as \fBcell\fP. The layout is determined by the flags \fBeCVExpandCols\fP and \fBeCVExpandRows\fP (see \fIenum CollViewOptions\fP) as well as the \fIinstvar cols\fP and the \fIinstvar rows\fP which all can be specified in the constructor. Three main types of layouts are possible:
.na
.nf

1. number of columns is fixed; CollectionView grows vertically
	- either set the flag \fBeCVExpandRows\fP
	- or set the \fIinstvar rows\fP to 0

2. number of rows is fixed; CollectionView grows horizontally
	- either set the flag \fBeCVExpandCols\fP
	- or set the \fIinstvar cols\fP to 0

3. square layout; CollectionView grows both horizontally and vertically
	- either set the flags \fBeCVExpandCols\fP and \fBeCVExpandRows\fP
	- or set both the \fIinstvar cols\fP and the \fIinstvar rows\fP to 0

.ad b
.fi
The items in the \fIinstvar coll\fP are aligned column first. For example, in a two column layout the items would be arranged like this:
.na
.nf

	1	4
	2	5
	3	6

.ad b
.fi
The layout can further be customized by modifying the \fIinstvar gap\fP and the \fIinstvar minExtent\fP. The gap is the space to reserve between two cells. The minimum extent contains the minimum size of a single cell. These values can be set with the \fImethod SetGap\fP, respectively the \fImethod SetMinExtent\fP.
.br
Generally, if part of the layout specification is changed the \fImethod Modified\fP must be called in order to recalculate the layout and make the change effective.
.br
To ease the access to the items \fBCollectionView\fP provides several methods to map columns and rows or geometrical coordinates to items or item rectangles and vice versa.
.br

.br
\fBSelecting Items\fP
.br

.br
Single items can be selected with the left mouse button. While the mouse button is pressed and moved the \fImethod DoOnItem\fP is continuously called. It forwards the message to the currently selected item, by default. When the mouse button is released the \fImethod DoSelect\fP is called. The default implementation informs the next event handlers via the \fImethod Control\fP about which item has been selected and whether a single or double click has been used.
.br
The \fIinstvar selection\fP stores the column and row numbers of the selected items. Clients may actively modify the current selection through the \fImethod SetSelection\fP or the \fImethod SetNoSelection\fP.

.br
classes are rarely derived from \fBCollectionView\fP.
.br
class \fBCollectionView\fP
is often reused directly.
.br
class \fBCollectionView\fP contains 36 methods.
.PD 0
.TP 20
\fBowner of class:\fR
nobody.
.PD 0
.TP 20
\fBbaseclasses:\fR
.na
View
.ad b
.PD 0
.TP 20
\fBsubclasses:\fR
.na
Menu, MenuBar
.ad b
.PD 0
.TP 20
\fBflags:\fR
CollViewOptions

.SH INSTANCE VARIABLES
.TP
.BR coll " (protected Collection *)"
contains the items displayed by the \fBCollectionView\fP. The objects in \fBcoll\fP must be instances of the \fIclass VObject\fP. \fBcoll\fP is specified in the constructor and can be set through the \fImethod SetCollection\fP as well. \fBcoll\fP is returned by the \fImethod GetCollection\fP. The destructor always destroys \fBcoll\fP and all contained items whereas the \fImethod SetCollection\fP allows to control whether to free the old collection.
.br
The \fBCollectionView\fP registers itself as an observer of \fBcoll\fP and automatically reformats its layout whenever \fBcoll\fP is modified. See \fImethod DoObserve\fP and \fImethod PrintOnWhenObserved\fP.

.TP
.BR cols " (protected short)"
contains the number of columns of the layout. \fBcols\fP is specified in the constructor. If the initial value is \fB\<= 0\fP or the flag \fBeCVExpandCols\fP (see \fIenum CollViewOptions\fP) is set, then \fBcols\fP is computed by the \fImethod Update\fP, otherwise the number of columns is fixed.
.br
See also \fIinstvar rows\fP.

.TP
.BR defaultItem " (protected VObject *)"
is the item returned by the \fImethod GetItem\fP if there is no object at the specified cell. By default, it is an instance of the \fIclass VObject\fP (!), but it can be set with the \fImethod SetDefaultItem\fP.
.br
The \fBdefaultItem\fP is only important when the \fBCollectionView\fP has empty cells. If the layout consists of more than one column and more than one row, then the number of items in the \fIinstvar coll\fP may be less than the number of available cells. In this case \fBGetItem\fP returns the \fBdefaultItem\fP for the empty cells. If an item in such a layout is selected the client has to check whether it is the \fBdefaultItem\fP.
.br
.B Notice:


.TP
.BR gap " (protected Point)"
is the thickness of the border around single items. In other words, the space between two items is twice the \fBgap\fP. The default value is zero but it can be modified with the \fImethod SetGap\fP.

.TP
.BR menu " (protected Menu *)"
is the popup menu of the \fBCollectionView\fP. By default, the \fBmenu\fP is \fBNULL\fP and the \fImethod GetMenu\fP returns the menu of the next handler. A specific menu can be set with the \fImethod SetMenu\fP.

.TP
.BR minExtent " (protected Point)"
contains the minimum size of a single cell. The default value is zero but it can be set with the \fImethod SetMinExtent\fP. The actual extent of cells within a row or column is calculated by the \fImethod Update\fP and depends on the minimum sizes of the VObjects in that row or column.

.TP
.BR rows " (protected short)"
contains the number of rows of the layout. \fBrows\fP is specified in the constructor. If the initial value is \fB\<= 0\fP or the flag \fBeCVExpandRows\fP (see \fIenum CollViewOptions\fP) is set, then \fBrows\fP is computed by the \fImethod Update\fP, otherwise the number of rows is fixed.
.br
See also \fIinstvar cols\fP.

.TP
.BR selection " (protected Rectangle)"
contains the currently selected cells. \fBselection\fP is accessible through the \fImethod GetSelection\fP. The respective components of the \fBRectangle\fP do not contain geometrical information but are interpreted this way:
.na
.nf

	\fBorigin.x\fP    column number of first selected cell
	\fBorigin.y\fP    row number of first selected cell
	\fBextent.x\fP    number of columns selected to the right
	\fBextent.y\fP    number of rows selected to the bottom

.ad b
.fi
Whether there is a selection at all can be checked with \fBGetSelection().IsEmpty()\fP. Selected items can be accessed by passing the column and row number to the \fImethod GetItem\fP.
.br
If the left mouse button is pressed (see \fImethod DoLeftButtonDownCommand\fP), then always only one cell can be selected. With the \fImethod SetSelection\fP a client can select several cells concurrently. The \fImethod SetNoSelection\fP clears the current selection.
.br
See also \fImethod DoSelect\fP.

.TP
.BR xPos " (protected short *)"
is an array containing the horizontal position of each column. These values are calculated by the \fImethod Update\fP and are used together with the \fIinstvar yPos\fP for layout management, drawing the grid and finding items.

.TP
.BR yPos " (protected short *)"
is an array containing the vertical position of each row. See \fIinstvar xPos\fP.


.SH INSTANCE METHOD LIST

.PD 0
.TP 20
client interface
CollectionView
.br
GetCollection
.br
SetCollection
.br
SetDefaultItem

.PD 0
.TP 20
drawing
Draw
.br
DrawGrid

.PD 0
.TP 20
implementation
~CollectionView

.PD 0
.TP 20
layout
GetMinSize
.br
SetGap
.br
SetMinExtent
.br
SetOrigin
.br
Update

.PD 0
.TP 20
mapping
GetItem
.br
InvalidateItemRect
.br
ItemPos
.br
ItemRect
.br
PointToItem

.PD 0
.TP 20
menus
DoCreateMenu
.br
GetMenu
.br
SetMenu

.PD 0
.TP 20
miscellaneous
Parts
.br
PrintOn
.br
PrintOnWhenObserved
.br
ReadFrom

.PD 0
.TP 20
scrolling
ConstrainScroll
.br
DoKeyCommand

.PD 0
.TP 20
selection
DoLeftButtonDownCommand
.br
DoOnItem
.br
DoSelect
.br
DoSelect2
.br
GetSelection
.br
SetNoSelection
.br
SetSelection

.PD 0
.TP 20
updating
DoObserve
.br
Modified

.SH CATEGORIES

.na
Standard Views
.ad b

.SH FILES
.PD 0
.TP 20
declaration:
\fBCollView.h

.SH KNOWN PROBLEMS

The \fImethod DoLeftButtonDownCommand\fP creates instances of the \fIclass CellSelector\fP that does not allow to select more than one cell concurrently though the implementation of \fBCollectionView\fP would allow this.
.br
Other problems: see \fImethod DoSelect\fP, \fImethod Draw\fP and \fImethod ReadFrom\fP.
.SH HISTORY
joe@csesbg.at	Fri Jun 28 14:49 1991	instvars
.br
joe@csesbg.at	Mon Jul 01 17:11 1991	methods
.br
joe@csesbg.at	Mon Jul 01 21:10 1991	main description
.br
gil@csesbg.at	Tue Jul 09 14:54 1991	reviewed
.br
chris@csesbg.at	Thu Jul 11 20:32 1991	reviewed
.br
.so mann/CollectionView::CollectionView.n
.so mann/CollectionView::ConstrainScroll.n
.so mann/CollectionView::DoCreateMenu.n
.so mann/CollectionView::DoKeyCommand.n
.so mann/CollectionView::DoLeftButtonDownCommand.n
.so mann/CollectionView::DoObserve.n
.so mann/CollectionView::DoOnItem.n
.so mann/CollectionView::DoSelect.n
.so mann/CollectionView::DoSelect2.n
.so mann/CollectionView::Draw.n
.so mann/CollectionView::DrawGrid.n
.so mann/CollectionView::GetCollection.n
.so mann/CollectionView::GetItem.n
.so mann/CollectionView::GetMenu.n
.so mann/CollectionView::GetMinSize.n
.so mann/CollectionView::GetSelection.n
.so mann/CollectionView::InvalidateItemRect.n
.so mann/CollectionView::ItemPos.n
.so mann/CollectionView::ItemRect.n
.so mann/CollectionView::Modified.n
.so mann/CollectionView::Parts.n
.so mann/CollectionView::PointToItem.n
.so mann/CollectionView::PrintOn.n
.so mann/CollectionView::PrintOnWhenObserved.n
.so mann/CollectionView::ReadFrom.n
.so mann/CollectionView::SetCollection.n
.so mann/CollectionView::SetDefaultItem.n
.so mann/CollectionView::SetGap.n
.so mann/CollectionView::SetMenu.n
.so mann/CollectionView::SetMinExtent.n
.so mann/CollectionView::SetNoSelection.n
.so mann/CollectionView::SetOrigin.n
.so mann/CollectionView::SetSelection.n
.so mann/CollectionView::Update.n
.so mann/CollectionView::~CollectionView.n
