.po -3
.nh
.TH "CompositeVObject" n "automatically Thu Mar 21 20:11:40 1991" "ET++ version v2.2" "ET++ class description"
.SH NAME
CompositeVObject.short \- a VObject composed of other VObjects
.SH DESCRIPTION
\fBCompositeVObject\fP is the abstract base class for VObjects that are composed of several other \fBVObjects\fP.
.br
A \fBCompositeVObject\fP is a \fBVObject\fP that contains a \fBCollection\fP of other VObjects. It can be treated as any other VObject and can itself be contained in another \fBCompositeVObject\fP, and so on. This tree-like organization is called the \fBVObject Tree\fP and is the way of building \fBcomplex window layouts\fP.
.br
\fBCompositeVObject\fP stores its VObjects, which are commonly referred to as contained or \fBinner VObjects\fP, in a \fBCollection\fP (see \fIinstvar list\fP) and forwards most of the methods of the \fIclass VObject\fP to each VObject in that list. In addition it provides several Collection methods in order to ease the access to the contained VObjects, such as Add, Remove, MakeIterator or Size which are passed to the \fIinstvar list\fP.
.br
\fBCompositeVObject\fP performs only a minimum layout calculation. Specific layout management is up to its subclasses by overriding \fImethod GetMinSize\fP, \fImethod Base\fP, \fImethod SetOrigin\fP and \fImethod SetExtent\fP. The \fIclass Expander\fP is a good example for this task.

.br
classes are always derived from \fBCompositeVObject\fP.
.br
class \fBCompositeVObject\fP
is never reused directly.
.br
class \fBCompositeVObject\fP is abstract.
.br
class \fBCompositeVObject\fP contains 32 methods.
.PD 0
.TP 20
\fBowner of class:\fR
nobody.
.PD 0
.TP 20
\fBbaseclasses:\fR
.na
VObject
.ad b
.PD 0
.TP 20
\fBsubclasses:\fR
.na
AbstractBorderItem, BackgroundItem, Button, Cluster, CycleItem, Expander, PullDownItem, Scroller, ShadowItem, Slider, Splitter, StretchWrapper, TreeNode
.ad b
.PD 0
.TP 20
\fBflags:\fR
CompositeVObjectFlags

.SH INSTANCE VARIABLES
.TP
.BR list " (protected Collection *)"
is the list of inner VObjects. It is initialized with an instance of the \fIclass OrdCollection\fP by the constructors or when the first \fBVObject\fP is added. The \fBlist\fP is deleted by the destructor.
.br
There are several methods to access and modify the \fBlist\fP such as the \fImethod Add\fP, \fImethod Remove\fP or \fImethod Size\fP. Whenever a VObject is added to or removed from the \fBlist\fP the \fIinstvar modified\fP is set to \fBTRUE\fP.

.TP
.BR modified " (protected bool)"
indicates whether the \fIinstvar list\fP has been modified. \fBmodified\fP is initially \fBTRUE\fP and set whenever VObjects are added or removed by the \fImethod Add\fP and the \fImethod Remove\fP. In addition, if a client registers a \fBCompositeVObject\fP as an observer of its \fIinstvar list\fP, then the \fBCompositeVObject\fP is also notified through the \fImethod DoObserve\fP so that the \fIinstvar list\fP can be modified by any other object.
.br
The \fBmodified\fP flag is used by subclasses to save time in layout calculation. For example, the \fIclass Cluster\fP, whose layout calculation is rather expensive, saves its minimum size in an instance variable and recalculates it only when the \fIinstvar list\fP has been modified. By calling the \fImethod SetModified\fP this recalculation can be forced.


.SH INSTANCE METHOD LIST

.PD 0
.TP 20
change propagation
DoObserve

.PD 0
.TP 20
client interface
CompositeVObject
.br
GetList

.PD 0
.TP 20
collection
Add
.br
At
.br
FreeAll
.br
SetAt
.br
SetModified

.PD 0
.TP 20
forwarding
Detect
.br
DispatchEvents
.br
Draw
.br
Enable
.br
GetMinSize
.br
MakeIterator
.br
Open
.br
Outline2
.br
Remove
.br
SendDown
.br
SetContainer
.br
Size

.PD 0
.TP 20
implementation
InspectorId
.br
Parts
.br
PrintOn
.br
ReadFrom
.br
~CompositeVObject

.PD 0
.TP 20
miscellaneous
SetItems

.PD 0
.TP 20
overrider interface
Base
.br
CompositeVObject
.br
SetExtent
.br
SetItems
.br
SetOrigin

.SH CATEGORIES

.na
Graphic Foundation
.ad b

.SH FILES
.PD 0
.TP 20
declaration:
\fBVObject.h

.SH HISTORY
joe@csesbg.at	Thu Jun 20 21:42 MET DST 1991	1st version
.br
chris@csesbg.at	Fri Jun 21 13:32 MET DST 1991	reviewed
.br
joe@csesbg.at	Fri Jun 21 14:11 MET DST 1991	last changes
.br
