.po -3
.nh
.TH "CycleItem" n "automatically Thu Mar 21 20:11:04 1991" "ET++ version v2.2" "ET++ class description"
.SH NAME
CycleItem \- cycles through several VObjects
.SH DESCRIPTION
The \fBCycleItem\fP cycles through a list of VObjects maintaining a current item. Only that item is drawn and gets events. Any kind of \fBVObject\fP can be put into a \fBCycleItem\fP, even more complex items such as a \fBScroller\fP or a \fBCluster\fP.
.br
The \fBCycleItem\fP itself should be put into an \fBEnumItem\fP because it reacts only on the \fBEnumItem\fP's up and down events. In addition, clients can actively set the current item with the \fImethod SetCurrentItem\fP.
.br
Example:
.na
.nf

VObject *layout =
        new EnumItem(cIdNone, eVObjVTop,
                new CycleItem(cIdNone, eVObjVBase,
                        new LabeledButton(cIdButton1, "on/off", FALSE),
                        new Scroller(new TextView(/* ... */)),
                        new OneOfCluster(cIdRadioBox, eVObjHLeft, gPoint8,
                                new LabeledButton(cIdBox1, "1"),
                                new LabeledButton(cIdBox2, "2"),
                                new LabeledButton(cIdBox3, "3"),
                                NULL
                        ),
                        NULL
                )
        );

.ad b
.fi


.br
classes are never derived from \fBCycleItem\fP.
.br
class \fBCycleItem\fP
is sometimes reused directly.
.br
class \fBCycleItem\fP contains 14 methods.
.PD 0
.TP 20
\fBowner of class:\fR
nobody.
.PD 0
.TP 20
\fBbaseclasses:\fR
.na
CompositeVObject
.ad b
.PD 0
.TP 20
\fBflags:\fR
CompositeVObjectFlags

.SH INSTANCE VARIABLES
.TP
.BR align " (protected VObjAlign)"
specifies how to align the current item within the content rectangle of the \fBCycleItem\fP. See also \fIenum VObjAlign\fP.

.TP
.BR current " (protected int)"
the index of the current item. Only that item is drawn and gets events. \fBcurrent\fP can be accessed with the \fImethod GetCurrentItem\fP and the \fImethod SetCurrentItem\fP.

.TP
.BR mode " (protected bool)"
\fBmode\fP affects layout calculation.
.br
If \fBmode\fP is \fBTRUE\fP, then each up and down cycle event triggers the recalculation of the overall dialog layout in order to adjust the size of the \fBCycleItem\fP to the size of the current item.
.br
If \fBmode\fP is \fBFALSE\fP, then the calculation is done only once, but the \fBCycleItem\fP will always have the size of its biggest item.
.br
Consider that setting the \fBmode\fP to \fBTRUE\fP will slow down execution speed.
.br
.B Known Problems:
Clients actually cannot set the \fBmode\fP because there is neither an access method nor a constructor argument. Only subclasses can set it.


.SH INSTANCE METHOD LIST

.PD 0
.TP 20
client interface
CycleItem
.br
GetCurrentItem
.br
SetCurrentItem

.PD 0
.TP 20
implementation
DispatchEvents
.br
Draw
.br
GetMinSize
.br
Init
.br
PrintOn
.br
ReadFrom
.br
SendDown
.br
SetExtent
.br
SetOrigin

.SH CATEGORIES

.na
VObject Composition
.ad b

.SH FILES
.PD 0
.TP 20
declaration:
\fBCycleItem.h

.SH HISTORY
joe@csesbg.at	Thu Sep 12 13:04 1991 MET DST
.br
.so mann/CycleItem::CycleItem.n
.so mann/CycleItem::DispatchEvents.n
.so mann/CycleItem::Draw.n
.so mann/CycleItem::GetCurrentItem.n
.so mann/CycleItem::GetMinSize.n
.so mann/CycleItem::Init.n
.so mann/CycleItem::PrintOn.n
.so mann/CycleItem::ReadFrom.n
.so mann/CycleItem::SendDown.n
.so mann/CycleItem::SetCurrentItem.n
.so mann/CycleItem::SetExtent.n
.so mann/CycleItem::SetOrigin.n
