.po -3
.nh
.TH "Dialog" n "automatically Fri Mar 15 14:08:59 1991" "ET++ version v2.2" "ET++ class description"
.SH NAME
Dialog \- undoable dialog boxes
.SH DESCRIPTION
The abstract class \fBDialog\fP implements \fBmodeless and modefull\fP dialog boxes. A \fBDialog\fP installs itself in a window and automatically closes that window whenever an \fBActionButton\fP is selected. \fBDialog\fP additionally defines a framework for \fBundoable\fP dialog boxes. It provides several abstract methods to save the current state of the dialog items when the dialog is opened, to restore the original state when the dialog is cancelled and to store the modified data when the dialog is accepted.
.br
The layout of a dialog box is the responsibility of subclasses which have to override the abstract \fImethod DialogView::DoCreateDialog\fP in order to create the dialog tree.
.br

.br
\fBModefull vs. Modeless\fP
.br

.br
Opening a modefull dialog preempts any input in another window of the application until the dialog is closed again. Modefull dialog windows are used, for example, by the \fIclass Alert\fP and the \fIclass FileDialog\fP. Modeless dialogs, however, do not block other windows. An example for that is the "Find/Change" dialog (see \fIclass ChangeDialog\fP).
.br
Whether a dialog is modefull or modeless is specified in the \fBwindowFlags\fP argument of the constructor of \fBDialog\fP. If the flag \fBeBWinBlock\fP is set, then the dialog is modefull, otherwise it is modeless (see also \fIenum BWinFlags\fP).
.br

.br
\fBOpening a Dialog\fP
.br

.br
Dialogs are opened either with the \fImethod Show\fP or the \fImethod ShowOnWindow\fP. The first one opens the window at the position where the last mouse event occurred whereas the latter positions the window at the center of a \fBVObject\fP. Before the dialog is opened the actual position is additionally modified by the \fImethod GetInitialPos\fP. The default implementation returns the center of the default \fBActionButton\fP which means that after opening the dialog that button is always exactly located under the mouse cursor.
.br
If the dialog is modefull, then \fBShow\fP and \fBShowOnWindow\fP block until the dialog is accepted or cancelled and then return the id of the selected \fBActionButton\fP.
.br
In case of a modeless dialog \fBShow\fP and \fBShowOnWindow\fP immediately return after opening the dialog window without waiting for events. This means that the returned id cannot be used. The \fImethod Control\fP has to be overridden and events from dialog items must be processed there.
.br

.br
\fBDialog Setup\fP
.br

.br
The purpose of the \fImethod DoSetDefaults\fP is to \fBinitialize\fP the dialog items with some reasonable default values. The \fImethod DoSetup\fP has to \fBenable/disable\fP the dialog items and initialize their values according to the data shown in the dialog. \fBDoSetup\fP is called whenever the dialog is opened whereas \fBDoSetDefaults\fP is called only when the dialog is opened the first time. Both methods are also invoked if a dialog item with the id \fIconst cIdDefault\fP is selected.
.br

.br
\fBValidation, Acceptation, Cancellation\fP
.br

.br
If an \fBActionButton\fP, whose id is different from \fIconst cIdCancel\fP, is selected, then the dialog is validated. \fBValidation\fP means that all dialog items check whether the data entered by the user can be accepted; e.g. the \fIclass NumItem\fP tests whether the entered number is in a certain range. If validation fails an error is reported and the dialog window stays open.
.br
A dialog is \fBaccepted\fP when the \fBOk\fP button (an \fBActionButton\fP with the id \fIconst cIdOk\fP) is selected and validation is successful. The data entered by the user is then stored by the \fImethod DoStore\fP and the dialog window is closed.
.br
\fBCancellation\fP is triggered by the \fBCancel\fP button (an \fBActionButton\fP with the id \fIconst cIdCancel\fP). The data entered by the user is discarded. The original data, which has been saved by the \fImethod DoSave\fP before opening the dialog window, is restored through the \fImethod DoRestore\fP and the dialog window is closed.
.br
Validation, acceptation and cancellation is performed by the \fImethod Control\fP.

.br
classes are always derived from \fBDialog\fP.
.br
class \fBDialog\fP
is never reused directly.
.br
class \fBDialog\fP is abstract.
.br
class \fBDialog\fP contains 15 methods.
.PD 0
.TP 20
\fBowner of class:\fR
nobody.
.PD 0
.TP 20
\fBbaseclasses:\fR
.na
DialogView
.ad b
.PD 0
.TP 20
\fBsubclasses:\fR
.na
Alert, ApplDialog, FileDialog, FindDialog, GotoDialog, PrintDialog, Printer
.ad b
.PD 0
.TP 20
\fBflags:\fR
ViewFlags

.SH INSTANCE VARIABLES
.TP
.BR actionId " (protected int)"
contains the id of the \fBActionButton\fP that caused the dialog window to be closed. The \fBactionId\fP is returned by the \fImethod Show\fP and the \fImethod ShowOnWindow\fP and indicates whether the dialog has been accepted (\fIconst cIdOk\fP, \fIconst cIdYes\fP) or cancelled (\fIconst cIdCancel\fP).

.TP
.BR dw " (protected BlankWin *)"
contains the dialog window created in the constructor. It is either an instance of the \fIclass BlankWin\fP or the \fIclass Window\fP depending on whether a window title is specified. The \fBDialog\fP installs itself in this window. See \fImethod Dialog\fP.


.SH INSTANCE METHOD LIST

.PD 0
.TP 20
client interface
Close
.br
SetTitle
.br
Show
.br
ShowOnWindow

.PD 0
.TP 20
implementation
Dialog
.br
ShowAt
.br
~Dialog

.PD 0
.TP 20
miscellaneous
InspectorId

.PD 0
.TP 20
overrider interface
Control
.br
DoRestore
.br
DoSave
.br
DoSetDefaults
.br
DoSetup
.br
DoStore
.br
GetInitialPos

.SH CATEGORIES

.na
Dialogs
.ad b

.SH FILES
.PD 0
.TP 20
declaration:
\fBDialog.h

.SH KNOWN PROBLEMS

See \fImethod ShowAt\fP.
.SH HISTORY
joe@csesbg.at	Fri Apr  5 16:07 1991
.br
joe@csesbg.at	Thu Jun 27 18:25 1991	instvars, methods
.br
joe@csesbg.at	Wed Jul 03 13:04 1991	main description
.br
chris@csesbg.at	Thu Jul 11 18:22 1991	reviewed
.br
.so mann/Dialog::Close.n
.so mann/Dialog::Control.n
.so mann/Dialog::Dialog.n
.so mann/Dialog::DoRestore.n
.so mann/Dialog::DoSave.n
.so mann/Dialog::DoSetDefaults.n
.so mann/Dialog::DoSetup.n
.so mann/Dialog::DoStore.n
.so mann/Dialog::GetInitialPos.n
.so mann/Dialog::InspectorId.n
.so mann/Dialog::SetTitle.n
.so mann/Dialog::Show.n
.so mann/Dialog::ShowAt.n
.so mann/Dialog::ShowOnWindow.n
.so mann/Dialog::~Dialog.n
