.po -3
.nh
.TH "DialogView" n "automatically Fri Mar 15 14:08:52 1991" "ET++ version v2.2" "ET++ class description"
.SH NAME
DialogView.short \- standard dialog behavior
.SH DESCRIPTION
The abstract class \fBDialogView\fP implements standard dialog behavior. It maintains an \fBactive text item\fP and allows to cycle through all editable text items with the \fBTab key\fP. In addition it identifies a \fBdefault button\fP which can be activated with the \fBReturn key\fP.
.br
Clients rarely create a direct subclass of \fBDialogView\fP. The abstract \fIclass Dialog\fP is commonly used as base class instead. A \fBDialog\fP is a \fBDialogView\fP that installs itself in a window and provides undoable dialog boxes. Neither \fBDialogView\fP nor \fBDialog\fP create a default dialog layout, so subclasses in any case have to override the abstract \fImethod DoCreateDialog\fP (see "Creating the Dialog Layout" below).
.br

.br
\fBCreating the Dialog Layout\fP
.br

.br
The layout of a dialog is a \fBVObject\fP tree which is usually referred to as \fBdialog tree\fP or \fBdialog layout\fP and the components of a dialog layout are called \fBdialog items\fP or simply items. The actual layout is up to subclasses which have to override the abstract \fImethod DoCreateDialog\fP to create and return the dialog tree. \fBDoCreateDialog\fP is called once when the dialog is opened the first time. The created layout is stored in the \fIinstvar dialogRoot\fP.
.br
A dialog layout contains simple, \fBoutput only items\fP, e.g. the \fIclass TextItem\fP and the \fIclass ImageItem\fP, items for \fBgrouping and aligning\fP of several items such as the \fIclass Form\fP, the \fIclass Cluster\fP, the \fIclass Expander\fP and \fBinput sensitive items\fP such as the \fIclass ActionButton\fP, \fIclass LabeledButton\fP, \fIclass ToggleButton\fP, \fIclass RadioButton\fP and the \fIclass PopupItem\fP. The \fIclass OneOfCluster\fP and the \fIclass ManyOfCluster\fP are special Clusters implementing a "one of" and "many of" behavior of their contained buttons. The \fIclass EditTextItem\fP provides an editable text line; the \fIclass NumItem\fP allows to enter a number.
.br
Good examples for creating a dialog layout can be found in the \fImethod ChangeDialog::DoCreateDialog\fP, the \fImethod FileDialog::DoCreateDialog\fP and the \fImethod PrintDialog::DoCreateDialog\fP.
.br

.br
\fBHandling Events from Dialog Items\fP
.br

.br
When creating the dialog layout \fBunique ids\fP must be assigned to \fBall selectable items\fP. When the user selects an item the id of the item is passed to the \fImethod Control\fP. Clients that wish to be notified of that event must override \fBControl\fP and check the id. Ids that are not handled should always be passed to the base class.
.br
Predefined ids such as \fIconst cIdOk\fP or \fIconst cIdCancel\fP are reserved for special items, such as the Ok and Cancel buttons. User defined ids must be greater than \fIconst cIdFirstUser\fP.
.br
Examples can be found in the following methods: \fImethod ChangeDialog::Control\fP, \fImethod FileDialog::Control\fP, \fImethod PrintDialog::Control\fP.
.br

.br
\fBKeyboard Focus Management\fP
.br

.br
\fBDialogView\fP overrides the \fImethod DispatchEvents\fP and, regardless of the position of the mouse, distributes all keyboard events to the \fBcurrent keyboard focus item\fP (see \fIinstvar kbdFocus\fP). If \fBTab or Shift-Tab\fP is pressed, then the \fImethod DoTab\fP is called which assigns the keyboard focus to the next, respectively previous keyboard focus item. An item may also actively grab the keyboard focus, e.g. when the user selects the item with the mouse. \fBDialogView\fP stores all items that are interested in the keyboard focus, in the \fIinstvar kbdFocusItems\fP. One of these items is the current keyboard focus item which is additionally referenced by the \fIinstvar kbdFocus\fP.
.br
Any kind of \fBVObject\fP may request the keyboard focus, but currently only the \fIclass EditTextItem\fP and its subclasses follow this protocol of \fBDialogView\fP. See the \fImethod EditTextItem::SetContainer\fP and the \fImethod EditTextItem::SendDown\fP for details.
.br
If the user presses the \fBReturn key\fP anywhere in the dialog, then the \fBdefault button\fP (usually the \fBOk button\fP) is selected. The \fIinstvar defaultButton\fP holds the default button and is initialized by a special instance of the \fIclass ActionButton\fP when the container chain of the window is set up. See the \fImethod ActionButton::SetContainer\fP.
.br
Clients may face a problem if the dialog layout contains an item that does not conform to the keyboard focus protocol of \fBDialogView\fP but should process keyboard events. For example, the dialog tree of a \fBFileDialog\fP contains a \fBCollectionView\fP. On key events it scrolls to the next item whose file name starts with the typed letter. Normally, it would not get any keyboard events, but \fBFileDialog\fP overrides the method \fBDispatchEvents\fP and passes key events explicitly to the \fBCollectionView\fP. See the \fImethod FileDialog::DispatchEvents\fP for details.

.br
classes are always derived from \fBDialogView\fP.
.br
class \fBDialogView\fP
is never reused directly.
.br
class \fBDialogView\fP is abstract.
.br
class \fBDialogView\fP contains 31 methods.
.PD 0
.TP 20
\fBowner of class:\fR
nobody.
.PD 0
.TP 20
\fBbaseclasses:\fR
.na
View
.ad b
.PD 0
.TP 20
\fBsubclasses:\fR
.na
Dialog, TreeView
.ad b
.PD 0
.TP 20
\fBflags:\fR
ViewFlags

.SH INSTANCE VARIABLES
.TP
.BR defaultButton " (protected Button *)"
refers to the button that is selected when the user presses the \fBReturn\fP key. The \fBdefaultButton\fP is a special instance of the \fIclass ActionButton\fP usually with the label "Ok". When the container chain of the dialog tree is initialized the default \fBActionButton\fP sets itself as the default button (see \fImethod ActionButton::SetContainer\fP). The \fBdefaultButton\fP can be accessed with the \fImethod GetDefaultButton\fP and is set with the \fImethod SetDefaultButton\fP.

.TP
.BR dialogRoot " (protected VObject *)"
is the root of the \fBVObject\fP tree containing the layout of the dialog. It is commonly referred to as \fBdialog tree\fP or \fBdialog layout\fP. When the dialog is opened the first time the dialog tree is created by the \fImethod DoCreateDialog\fP. The \fBdialogRoot\fP can be accessed with the \fImethod GetRoot\fP and replaced with a new one through the \fImethod SetDialog\fP. The whole dialog tree is recursively destroyed in the destructor (by means of \fBFreeAll\fP).

.TP
.BR kbdFocus " (protected VObject *)"
is the dialog item which currently has the keyboard focus (usually an instance of the \fIclass EditTextItem\fP). The \fBDialogView\fP passes all keyboard and right mouse button events (for the popup menu) to the \fBkbdFocus\fP. If the \fBTab\fP is pressed, then the keyboard focus is assigned to another item through the \fImethod DoTab\fP. The \fBkbdFocus\fP is always an element of the \fIinstvar kbdFocusItems\fP. It can be accessed with the \fImethod GetKbdFocus\fP and set with the \fImethod SetKbdFocus\fP.

.TP
.BR kbdFocusItems " (protected OrdCollection *)"
contains all dialog items that are interested in getting the keyboard focus. These items are usually instances of the \fIclass EditTextItem\fP but may be any kind of \fBVObject\fP. The current keyboard focus item is contained in the \fIinstvar kbdFocus\fP. When the user presses \fBTab or Shift-Tab\fP, then the next, respectively the previous enabled item in this list gets the keyboard focus (see \fImethod DoTab\fP). During container initialization of a new dialog tree the dialog items add themselves to this list through the \fImethod AddItemWithKbdFocus\fP. See also \fImethod EditTextItem::SetContainer\fP.

.TP
.BR menu " (protected Menu *)"
is the popup menu of this dialog. The \fImethod GetMenu\fP creates the \fBmenu\fP only if there is no next event handler.
.br
.B Known Problems:
The \fBmenu\fP is not deleted in the destructor.

.TP
.BR modified " (protected bool)"
indicates whether the layout of the dialog tree has changed. If \fBmodified\fP is \fBTRUE\fP, then the layout is recalculated by the \fImethod Update\fP next time the dialog is redrawn. By default, the \fBDialogView\fP never sets \fBmodified\fP, except in the constructor. The flag \fBmodified\fP can be tested with the \fImethod IsModified\fP.


.SH INSTANCE METHOD LIST

.PD 0
.TP 20
client interface
AddItemWithKbdFocus
.br
DisableItem
.br
EnableItem
.br
FindItem
.br
GetDefaultButton
.br
GetKbdFocus
.br
GetRoot
.br
IsModified
.br
RemoveItemWithKbdFocus
.br
SetDefaultButton

.PD 0
.TP 20
forwarding
DispatchEvents
.br
DoMenuCommand
.br
Open
.br
SetContainer
.br
SetExtent
.br
SetOrigin

.PD 0
.TP 20
implementation
CalcLayout
.br
DialogView
.br
DoTab
.br
Draw
.br
FindNeighbours
.br
GetMenu
.br
GetMinSize
.br
HasSelection
.br
SetDialog
.br
SetKbdFocus
.br
Update
.br
~DialogView

.PD 0
.TP 20
miscellaneous
Parts

.PD 0
.TP 20
overrider interface
Control
.br
DoCreateDialog

.SH CATEGORIES

.na
Standard Views
.ad b

.SH FILES
.PD 0
.TP 20
declaration:
\fBDialog.h

.SH KNOWN PROBLEMS

See \fIinstvar menu\fP, \fImethod FindItem\fP and \fImethod SetDialog\fP.
.SH HISTORY
joe@csesbg.at	Fri Apr  5 16:07 1991
.br
joe@csesbg.at	Wed Jun 26 15:32 1991	instvars
.br
joe@csesbg.at	Wed Jun 26 18:45 1991	methods
.br
joe@csesbg.at	Wed Jul 10 18:50 1991	main description
.br
chris@csesbg.at	Thu Jul 11 18:58 1991	reviewed
.br
