.po -3
.nh
.TH "Document" n "automatically Fri Mar 15 14:13:05 1991" "ET++ version v2.2" "ET++ class description"
.SH NAME
Document.short \- loading/storing data, performing commands, handling dialogs
.SH DESCRIPTION
The abstract class \fBDocument\fP manages the data of an application. Documents are created and opened by the application (see \fIclass Application\fP). \fBDocument\fP maintains a document type and provides generic functionality for loading and storing the data, managing windows, executing user commands, keeping track of changes made to the data and displaying the necessary alerts and file dialogs.
.br

.br
\fBLoading and Storing\fP
.br

.br
The document's data is loaded from and stored into a file. \fBDocument\fP creates and opens the file when necessary and then calls the \fImethod DoRead\fP and the \fImethod DoWrite\fP. These methods must be overridden to read and write the document's data. The Object Input/Output mechanism is normally used for reading and writing (see \fIcookbook Object Input/Output\fP).
.br
It is also possible to import data from another file into the document. To use this the \fImethod CanImportDocument\fP and the \fImethod DoImport\fP must be overridden. The first method has to check whether the file can be imported at all and the second has to do the actual work.
.br

.br
\fBDocument Type\fP
.br

.br
Every document has a type which is stored in the \fIinstvar docType\fP. The type is written to the file when the document is saved. This allows to determine the document type of a file when it is reopened later. The type is passed to the \fImethod CanOpenDocument\fP and the \fImethod CanImportDocument\fP to check whether the file can be opened or imported.
.br

.br
\fBWindows and Icon\fP
.br

.br
Each document has a main window. The main window must be created by the \fImethod DoMakeWindows\fP and is added to the document's list of windows in the \fIinstvar windows\fP. The layout of the main window usually contains a view to present the document's data to the user. For example, a text editor would have a \fBTextView\fP installed in the main window. Other windows which logically belong to the document can be added to the list with the \fImethod AddWindow\fP, e.g. the "Find/Change" dialog.
.br
A document can be iconified and deiconified by the \fImethod Toggle\fP. This method closes all the windows contained in the list and opens an icon window instead. The icon window is created by the \fImethod DoMakeIcon\fP.
.br

.br
\fBCommands and Change Management\fP
.br

.br
The \fImethod PerformCommand\fP executes commands. The last executed command is saved in the \fIinstvar lastCmd\fP so that it can be undone or redone later by the \fImethod Undo\fP. Whenever a command is performed or undone the document updates the \fIinstvar changeCount\fP. The change count indicates whether a document is modified and therefore has unsaved changes. For example, if a document has been modified and is to be closed, then an alert is popped up which allows the user to decide what to do with the changes.
.br


.br
classes are always derived from \fBDocument\fP.
.br
class \fBDocument\fP
is never reused directly.
.br
class \fBDocument\fP is abstract.
.br
class \fBDocument\fP contains 51 methods.
.PD 0
.TP 20
\fBowner of class:\fR
nobody.
.PD 0
.TP 20
\fBbaseclasses:\fR
.na
EvtHandler
.ad b
.PD 0
.TP 20
\fBsubclasses:\fR
.na
CmdHistDocument
.ad b
.PD 0
.TP 20
\fBflags:\fR
EvtFlags

.SH INSTANCE VARIABLES
.TP
.BR application " (protected Application *)"
represents the application this document belongs to. Whenever the application adds a document to its list it calls the \fImethod SetApplication\fP which sets this instance variable. \fBapplication\fP is returned as next event handler by the \fImethod GetNextHandler\fP. It is also used to call some document handling methods of the application.
The application is additionally accessible through the \fIvar gApplication\fP.

.TP
.BR changeCount " (protected int)"
contains the \fBnumber of changes\fP performed on this document since the last save operation. Documents are changed by command objects which are instances of the \fIclass Command\fP. The \fImethod PerformCommand\fP executes commands through the \fImethod Command::Do\fP which returns the actual number of changes caused by the command. This number is added to the current \fBchangeCount\fP and the new value is set with the \fImethod SetChangeCount\fP. The same happens when a command is "undone" with the exception that the \fImethod Command::Undo\fP returns a negative value so that the \fBchangeCount\fP is decremented. Whenever the \fImethod Load\fP loads a new document the \fBchangeCount\fP is initialized to 0 by the \fImethod InitChangeCount\fP.
The \fBchangeCount\fP should not be modified directly. It can be accessed with the \fImethod GetChangeCount\fP.

.TP
.BR docName " (protected char *)"
contains the \fBname of the document\fP. Unless the document is new and therefore "untitled" (see \fIinstvar isUntitled\fP) the \fBdocName\fP is equal to the name of the associated file. The \fBdocName\fP is accessed by the \fImethod GetName\fP and modified by the \fImethod SetName\fP. The \fImethod SetName\fP passes it to the \fImethod SetWindowTitle\fP and the \fBdocName\fP is also used as title for the \fIinstvar icon\fP. If the \fBdocName\fP contains only the file name, then the path can be accessed by the \fImethod GetLoadDir\fP or the \fIinstvar loadDir\fP.

.TP
.BR docType " (protected const char *)"
contains the \fBtype of the document\fP. It is initialized by the constructor. If an application handles only one type of documents, then the \fBdocType\fP is usually equal to the main document type of the application (see \fIclass Application\fP). Whenever a document is saved the \fImethod DoWrite\fP saves the \fBdocType\fP into the file so that the type can be checked later by the \fImethod CanLoadDocument\fP or the \fImethod Application::CanOpenDocument\fP.
.br
Document types are defined by constants, e.g. \fBconst char *cDocTypeMyType = "MYDOCTYPE";\fP. ET++ has a pre-defined set of document types (see \fIclass FileType\fP). For example, a document which handles pure ASCII files would set \fBdocType\fP to \fIconst cDocTypeAscii\fP.

.TP
.BR icon " (protected Icon *)"
is the \fBicon window\fP which is opened or closed when a document's windows are (de)iconified through the \fImethod Toggle\fP. It is initially \fBNULL\fP and is created by the \fImethod DoMakeIcon\fP when the document is iconified the first time (delayed creation). By default, the \fImethod DoMakeIcon\fP creates an instance of the \fIclass Icon\fP.

.TP
.BR isConverted " (protected bool)"
indicates whether there is a difference between the \fIinstvar docType\fP and the type of the file opened by the document. If the types are different it is assumed that the file's data must have been converted somehow during loading or will be converted before saving. After reading the data the \fImethod Load\fP finds out this difference and sets \fBisConverted\fP accordingly. Actually the types can differ only if the \fImethod CanLoadDocument\fP checks for other types than the \fIinstvar docType\fP.
.br
The purpose of all this is to avoid that a converted document overwrites the original file. \fBDocument\fP ensures this by calling the \fImethod SaveAs\fP instead of the \fImethod Save\fP if \fBisConverted\fP is set. See also \fIinstvar isUntitled\fP.

.TP
.BR isOpen " (protected bool)"
tells whether the document is still open or has already been closed. It is set by the constructor. The \fImethod Close\fP uses it to ensure that the document is closed and changes are saved only once.

.TP
.BR isUntitled " (protected bool)"
indicates whether the document is new and therefore "untitled". \fBisUntitled\fP is \fBTRUE\fP if the document has just been created and has not performed a load or save yet, e.g. a document created when the user selects the "new" button of the application window. The application initializes the name of an untitled document with the string "untitled.%d" where "%d" is the current number of untitled documents within the application.
.br
\fBDocument\fP calls the \fImethod SaveAs\fP instead of the \fImethod Save\fP if \fBisUntitled\fP is set because the user has to enter a file name before saving. See also \fIinstvar isConverted\fP.

.TP
.BR lastCmd " (protected Command *)"
contains the last executed command. Commands are executed the first time by the \fImethod PerformCommand\fP and undone or redone by the \fImethod Undo\fP. Immediately after a load or a save the undo mechanism is reset. The \fBlastCmd\fP is then set to \fBgResetUndo\fP. \fBlastCmd\fP initially contains \fBgNoChanges\fP.
\fBlastCmd\fP is directly modified by the \fIclass CmdHistDocument\fP.
.br
.B Known Problems:
See \fImethod ~Document\fP.

.TP
.BR loadDir " (protected char *)"
contains the path name of the loaded file. The full name of the file can therefore be constructed by appending a "/" and the \fIinstvar docName\fP to the \fBloadDir\fP. The \fBloadDir\fP is needed by the \fImethod Save\fP because the document name need not contain the full path name and the current working directory can change between loading and saving.

.TP
.BR makeBackup " (protected bool)"
controls the \fBcreation of backup files\fP. The \fImethod MakeBackup\fP, which is called before each save operation, creates a backup file if \fBmakeBackup\fP is set. It is initially \fBFALSE\fP but can be modified with the \fImethod EnableBackups\fP.

.TP
.BR menu " (protected Menu *)"
is the \fBpopup menu\fP returned by the \fImethod GetMenu\fP. The menu is created in the constructor with the title "edit". Menu items are added later by the \fImethod DoCreateMenu\fP immediately before the menu is shown the first time.

.TP
.BR uniqueId " (protected int)"
is a number which uniquely \fBidentifies the document\fP. This number is initialized to 0 in the constructor. Whenever a file is read or written an instance of the \fIclass FileType\fP is created which contains information about the file. Part of the information is a number (see \fImethod FileType::UniqueId\fP) which is copied to \fBuniqueId\fP and allows to identify the document within the application (see \fImethod Application::FindDocument\fP). The \fBuniqueId\fP can be accessed with the \fImethod UniqueId\fP.

.TP
.BR wasopen " (protected bool *)"
\fBwasopen\fP is unused.

.TP
.BR window " (protected Window *)"
contains the so called \fBmain window\fP of the document because it is the first opened window and usually displays the document's data. The main window is created by the \fImethod DoMakeWindows\fP and can be accessed through the \fImethod GetWindow\fP. It is also stored as first entry in the \fIinstvar windows\fP.

.TP
.BR windows " (protected ObjList *)"
is the document's \fBlist of windows\fP whereby the first entry is identical with the \fIinstvar window\fP, the document's main window. New windows can be added with the \fImethod AddWindow\fP. Typical windows added to this list are modeless dialogs, e.g. the "Find/Change" dialog window. Adding windows to this list is useful because they are included in (de)iconifying by the \fImethod Toggle\fP.
.br
\fBwindows\fP is initialized in the constructor with an instance of the \fIclass ObjList\fP. The destructor destroys \fBwindows\fP and the main window, which is the first entry in the list, but not the other windows contained in the list.


.SH INSTANCE METHOD LIST

.PD 0
.TP 20
change management
AlertChanges
.br
GetChangeCount
.br
InitChangeCount
.br
Modified
.br
SetChangeCount

.PD 0
.TP 20
commands
PerformCommand

.PD 0
.TP 20
constructor
Document

.PD 0
.TP 20
destructor
~Document

.PD 0
.TP 20
document attributes
GetDocumentType
.br
GetLoadDir
.br
GetName
.br
IsUntitled
.br
SetName
.br
UniqueId

.PD 0
.TP 20
event handling
SetApplication

.PD 0
.TP 20
icon
DoMakeIcon
.br
Toggle

.PD 0
.TP 20
input/output
CanImportDocument
.br
CanLoadDocument
.br
DoFileIsAlreadyOpen
.br
DoImport
.br
DoRead
.br
DoWrite
.br
EnableBackups
.br
Load
.br
MakeBackup

.PD 0
.TP 20
menu commands
Close
.br
Import
.br
Open
.br
Redo
.br
Revert
.br
Save
.br
SaveAs
.br
Undo

.PD 0
.TP 20
miscellaneous
Control
.br
DoCreateMenu
.br
DoMenuCommand
.br
DoSetupMenu
.br
GetMenu
.br
GetNextHandler
.br
InspectorId
.br
Parts

.PD 0
.TP 20
storing
Store

.PD 0
.TP 20
windows
AddWindow
.br
DoMakeViews
.br
DoMakeWindows
.br
GetWindow
.br
MakeFileDialog
.br
OpenWindows
.br
SetWindowTitle
.br
ShowDocument

.SH CATEGORIES

.na
Application Framework
.ad b

.SH FILES
.PD 0
.TP 20
declaration:
\fBDocument.h

.SH KNOWN PROBLEMS

The command \fBgResetUndo\fP should be executed by the \fImethod Load\fP and the \fImethod Store\fP themselves and not by their callers: \fImethod Open\fP, \fImethod Close\fP, \fImethod Save\fP and \fImethod SaveAs\fP. By the way, \fBPerformCommand(gResetUndo)\fP is missing in the \fImethod Revert\fP.
.SH HISTORY
joe - Mon Apr  8 11:44 MET DST 1991 - first version
.br
joe - Thu Jun  6 20:47 MET DST 1991 - SBG changes
.br
joe - Tue Jun 11 15:12 MET DST 1991 - changes of chris/gil
.br
