.po -3
.nh
.TH "EvtHandler" n "automatically Fri Mar 15 14:09:27 1991" "ET++ version v2.2" "ET++ class description"
.SH NAME
EvtHandler \- event handler chain; controlling, commands, menus
.SH DESCRIPTION
\fBEvtHandler\fP is the abstract base class for all objects that handle events, perform commands and deal with popup menus. The class \fBEvtHandler\fP introduces the \fBevent handler chain\fP concept (see also \fIcookbook Event Handling\fP).
.br

.br
\fBEvent Handler Chain\fP
.br

.br
Event handlers are organized in a chain. The link between two handlers is the \fImethod GetNextHandler\fP. Most of the methods defined by \fBEvtHandler\fP are implemented by forwarding the message to the next handler. Subclasses that override a method of \fBEvtHandler\fP should always call the inherited method in order to preserve the chain.
.br
As an example, the \fBVObject\fP tree (see \fIclass VObject\fP) of a window contains several event handler chains. Each chain starts at a leaf of the tree, follows the hierarchy upwards and either ends at the window or the window links it to another handler, usually a \fBDocument\fP. Another example is the event handler chain \fBView\fP -> \fBDocument\fP -> \fBApplication\fP.
.br
The event handler chain serves several purposes: Controlling, Command and Menu handling.
.br

.br
\fBControlling\fP
.br

.br
Input events from the keyboard and the mouse are distributed from the root of a VObject tree to the leafs. The leafs can generate additional, application-specific events. Such events are forwarded along the event handler chain via the \fImethod Control\fP. For example, things like buttons use \fBControl\fP to inform the "controlling" \fBDialog\fP about a selection event (see \fImethod Button::DoOnItem\fP, \fImethod ActionButton::Control\fP and \fImethod Dialog::Control\fP).
.br
The counterpart of \fBControl\fP is the \fImethod SendDown\fP (in older ET++ versions called \fBDownControl\fP). \fBSendDown\fP does not follow the event handler chain but allows a \fBVObject\fP to communicate with other objects that are lower in the \fBVObject\fP tree. By means of this method a \fBVObject\fP can request one or several objects down in the tree to do certain things or return some data. For example, the \fIclass OneOfCluster\fP uses \fBSendDown\fP to switch one of its contained buttons on and all others off. The \fImethod DialogView::Control\fP calls \fBSendDown\fP to assign the keyboard focus.
.br

.br
\fBCommands\fP (see also \fIcookbook Commands\fP)
.br

.br
The \fImethod VObject::DispatchEvents\fP distributes mouse and keyboard events to the corresponding \fBDo...Command\fP methods. The command object returned by these methods is passed along the event handler chain through the \fImethod PerformCommand\fP until the end of the chain is reached or an \fBEvtHandler\fP has overridden \fBPerformCommand\fP and executes the command. This is usually done by a \fBDocument\fP (see the \fImethod Document::PerformCommand\fP).
.br

.br
\fBMenus\fP (see also \fIcookbook Menus\fP)
.br

.br
Whenever the right mouse button is pressed over a \fBVObject\fP, then the a popup menu is shown. Four methods of \fBEvtHandler\fP deal with that menu: \fImethod GetMenu\fP, \fImethod DoCreateMenu\fP, \fImethod DoSetupMenu\fP and \fImethod DoMenuCommand\fP. By default, all methods are passed along the event handler chain so that several handlers may take part in the creation and setup of the menu as well as in the handling of menu commands.  The method \fIVObject::DoRightButtonDownCommand\fP calls the menu handling methods in the following order:
.na
.nf

	\fBGetMenu\fP		passed to the next handlers until one returns a menu
	\fBDoCreateMenu\fP		called once for new menus; allows to add menu items and submenus
	\fBDoSetupMenu\fP		enable/disable menu items and update their text
	\fBDoMenuCommand\fP	handle the menu command selected by the user

.ad b
.fi


.br
classes are always derived from \fBEvtHandler\fP.
.br
class \fBEvtHandler\fP
is never reused directly.
.br
class \fBEvtHandler\fP is abstract.
.br
class \fBEvtHandler\fP contains 12 methods.
.PD 0
.TP 20
\fBowner of class:\fR
nobody.
.PD 0
.TP 20
\fBbaseclasses:\fR
.na
Object
.ad b
.PD 0
.TP 20
\fBsubclasses:\fR
.na
Application, Document, VObject
.ad b
.PD 0
.TP 20
\fBflags:\fR
EvtFlags


.SH INSTANCE METHOD LIST

.PD 0
.TP 20
commands
PerformCommand

.PD 0
.TP 20
constructor
EvtHandler

.PD 0
.TP 20
event handling
Control
.br
DoIdleCommand
.br
GetNextHandler
.br
Send
.br
SendDown

.PD 0
.TP 20
menus
DoCreateMenu
.br
DoMenuCommand
.br
DoSetupMenu
.br
GetMenu

.PD 0
.TP 20
metaclass
FindNextHandlerOfClass

.SH CATEGORIES

.na
Application Framework
.ad b

.SH FILES
.PD 0
.TP 20
declaration:
\fBEvtHandler.h

.SH HISTORY
joe@csesbg.at	Thu Mar 28 16:25 MET 1991	1st version
.br
joe@csesbg.at	Mon Jun 17 19:39 MET DST 1991	reworked
.br
chris@csesbg.at	Tue Jun 18 20:30 MET DST 1991	reviewed
.br
joe@csesbg.at	Wed Jun 19 16:51 MET DST 1991	last changes
.br
.so mann/EvtHandler::Control.n
.so mann/EvtHandler::DoCreateMenu.n
.so mann/EvtHandler::DoIdleCommand.n
.so mann/EvtHandler::DoMenuCommand.n
.so mann/EvtHandler::DoSetupMenu.n
.so mann/EvtHandler::EvtHandler.n
.so mann/EvtHandler::FindNextHandlerOfClass.n
.so mann/EvtHandler::GetMenu.n
.so mann/EvtHandler::GetNextHandler.n
.so mann/EvtHandler::PerformCommand.n
.so mann/EvtHandler::Send.n
.so mann/EvtHandler::SendDown.n
