.po -3
.nh
.TH "RegularExp" n "automatically Mon Apr  8 20:18:26 1991" "ET++ version v2.2" "ET++ class description"
.SH NAME
RegularExp \- GNU emacs regular expressions
.SH DESCRIPTION
The class \fBRegularExp\fP implements \fBregular expressions\fP in the style of the GNU emacs editor. Additional information about GNU emacs regular expressions is contained in the file \fB$ET_DIR/doc/regex.txt\fP.
.br
Regular expressions are used to check whether a string matches or contains a certain \fBpattern\fP. For example, the pattern \fB[0-9]*\fP matches all strings that consist of any number of digits including zero.
.br
\fBRegularExp\fP provides two ways of comparing:
.br
It can be tested whether a string \fBexactly matches\fP the regular expression or whether the pattern is \fBcontained\fP somewhere in the string. The second way allows to find any occurrence of the pattern by \fBsearching forward or backward\fP in the string.
.br
The "Find/Change" dialog of a \fBTextView\fP allows to enter a regular expression in the "Find:" field. Regular expressions are also used by the \fIclass RestrTextView\fP and the \fIclass RestrTextItem\fP.
.br

.br
Examples for valid regular expressions:
.na
.nf

float:		"\(+\|-\)?[0-9]*\(\.[0-9]*\)?"
int:		"\(+\|-\)?[0-9]*"
double:		"\(+\|-\)?[0-9]*\(\.[0-9]*\)?\([eE]\(+\|-\)?[0-9]*\)?" 
alpha:		"[A-Za-z]+"
lowercase:	"[a-z]+"
uppercase:	"[A-Z]+"
alphanum:	"[0-9A-Za-z]+"
identifier:	"[A-Za-z_][A-Za-z0-9_]*"

.ad b
.fi


.br
classes are never derived from \fBRegularExp\fP.
.br
class \fBRegularExp\fP
is sometimes reused directly.
.br
class \fBRegularExp\fP contains 15 methods.
.PD 0
.TP 20
\fBowner of class:\fR
nobody.
.PD 0
.TP 20
\fBbaseclasses:\fR
.na
Object
.ad b
.PD 0
.TP 20
\fBflags:\fR
ObjFlags

.SH INSTANCE VARIABLES
.TP
.BR caseSensitive " (private bool)"
controls whether case is significant in searching

.TP
.BR fastSearch " (private bool)"
controls whether the fast map must be compiled for the regular expression

.TP
.BR pb " (private re_pattern_buffer *)"
a structured representation of the source pattern

.TP
.BR result " (private char *)"
the result of compiling the source pattern. It is \fBNULL\fP if the compilation was successful, otherwise it refers to an error message. \fBresult\fP can be accessed with the \fImethod GetExprState\fP.

.TP
.BR source " (private char *)"
the source pattern as specified in the constructor or in the \fImethod Reset\fP. It is accessible with the \fImethod GetPattern\fP.


.SH INSTANCE METHOD LIST

.PD 0
.TP 20
client interface
GetExprState
.br
GetPattern
.br
Match
.br
MatchWordPattern
.br
RegularExp
.br
Reset
.br
SearchBackward
.br
SearchForward

.PD 0
.TP 20
implementation
InspectorId
.br
Match2
.br
PrintOn
.br
ReadFrom
.br
SearchBackward2
.br
SearchForward2
.br
~RegularExp

.SH CATEGORIES

.na
Text Support
.ad b

.SH FILES
.PD 0
.TP 20
declaration:
\fBRegularExp.h

.SH HISTORY
joe@csesbg.at	Thu Sep 05 16:55 1991 MET DST
.br
.so mann/RegularExp::GetExprState.n
.so mann/RegularExp::GetPattern.n
.so mann/RegularExp::InspectorId.n
.so mann/RegularExp::Match.n
.so mann/RegularExp::Match2.n
.so mann/RegularExp::MatchWordPattern.n
.so mann/RegularExp::PrintOn.n
.so mann/RegularExp::ReadFrom.n
.so mann/RegularExp::RegularExp.n
.so mann/RegularExp::Reset.n
.so mann/RegularExp::SearchBackward.n
.so mann/RegularExp::SearchBackward2.n
.so mann/RegularExp::SearchForward.n
.so mann/RegularExp::SearchForward2.n
.so mann/RegularExp::~RegularExp.n
