.po -3
.nh
.TH "VObject" n "automatically Mon Apr  8 20:17:57 1991" "ET++ version v2.2" "ET++ class description"
.SH NAME
VObject.short \- base class for all visual objects drawn on the screen
.SH DESCRIPTION
\fBVObject\fP (visual object) is the most general graphical class in ET++ and, in this respect, corresponds to the \fIclass Object\fP in the overall class hierarchy. It defines the abstract protocol for managing the size of graphical shapes, for input handling and for drawing on the screen.
.br

.br
\fBMetric and Baseline\fP
.br

.br
A \fBVObject\fP is located at a certain position in a window and occupies some space thereof. This is called the \fBcontent rectangle\fP specified by its origin and extent. It is stored in the \fIinstvar contentRect\fP. The content rectangles of all \fBVObjects\fP contained in a window are calculated when the window is opened the first time and whenever it is resized. During the calculation of the window layout each \fBVObject\fP must calculate its own \fBminimum size\fP which is an instance of the \fIclass Metric\fP and which is returned by the \fImethod GetMinSize\fP. The available space for a \fBVObject\fP may be greater than its minimum size. In this case the flags \fBeVObjHFixed\fP and \fBeVObjVFixed\fP (see \fIenum VObjFlags\fP) control whether a \fBVObject\fP may be expanded in the respective direction.
.br
\fBVObjects\fP dealing with text are often aligned at their \fBbaseline\fP. This is the line below which letters like 'g' descend. The baseline must be calculated by the \fImethod Base\fP and also returned by the \fImethod GetMinSize\fP as part of the \fBMetric\fP (see above). For example, the \fIclass TextItem\fP determines its metric based on the used font and the string to be shown.
.br
See also \fItechnote Layout Management\fP.
.br

.br
\fBLayout and VObject Tree\fP
.br

.br
Complex window layouts are built by combining several \fBVObjects\fP into a single, composite object which can be treated as one \fBVObject\fP. The composite object can again be a part of another composite object, and so on. Therefore the \fBVObjects\fP in a window are organized in a tree-like fashion.
.br
The abstract \fIclass CompositeVObject\fP manages a list of \fBVObjects\fP and applies methods executed on itself to all of its components. The actual layout management is up to its subclasses. They have to override the \fImethod GetMinSize\fP, the \fImethod SetOrigin\fP and the \fImethod SetExtent\fP in order to divide the available space between their components; e.g. the \fIclass Cluster\fP aligns \fBVObjects\fP within rows and columns. Another example for a composite \fBVObject\fP is the \fIclass VObjectPair\fP that arranges two \fBVObjects\fP horizontally.
.br
See also \fItechnote Layout Management\fP.
.br

.br
\fBDrawing\fP
.br

.br
ET++ uses an \fBindirect drawing scheme\fP which consists of two steps: \fBinvalidation and drawing\fP. The application announces which objects must be redrawn by calling the \fImethod ForceRedraw\fP or the \fImethod InvalidateRect\fP. The window collects the invalidated areas and redraws them after the next input event is processed. The \fImethod UpdateEvent\fP can be used to force any pending update to be done immediately.
.br
The actual drawing must be done in the \fImethod Draw\fP which is called when the invalid regions of a window are updated. Drawing occurs on output ports and the \fIclass Port\fP provides a rich set of device-independent methods for graphics primitives such as lines, rectangles, ovals, polygons, bitmaps, and more.
.br
If a \fBVObject\fP is selected (e.g. a menu item), then it should be highlighted somehow in order to give a feedback to the user. \fBHighlighting\fP is done by the \fImethod DrawHighlight\fP which, by default, draws the \fBVObject\fP in a different color, respectively inverts it on monochrome screens.
.br
See also \fItechnote Invalidation and Drawing\fP and \fItechnote Drawing Model\fP.
.br

.br
\fBEvent Dispatching\fP
.br

.br
If the mouse cursor is located over a \fBVObject\fP, then this \fBVObject\fP gets all mouse and keyboard events. Events are distributed by the window with instances of the \fIclass Token\fP. The \fImethod DispatchEvents\fP examines the input token and dispatches it to the corresponding \fBDo...Command\fP methods, such as the \fImethod DoLeftMouseButtonDownCommand\fP or the \fImethod DoKeyCommand\fP. The default action of these methods is to propagate the event up in the VObject tree to the \fIinstvar container\fP. In order to react to a specific event the corresponding method must be overridden.
.br
See also \fItechnote Event Handling\fP.
.br

.br
\fBMenu\fP
.br

.br
When the right mouse button is pressed while the mouse cursor is over a \fBVObject\fP, then a popup menu is displayed at the current mouse position. The menu is created and menu commands can be processed by overriding the methods: \fImethod GetMenu\fP, \fImethod DoCreateMenu\fP, \fImethod DoSetupMenu\fP and \fImethod DoMenuCommand\fP. The default action of these methods is to forward the method to the next event handler which is the \fIinstvar container\fP, by default.
.br
See \fIcookbook Menus\fP and \fItechnote Event Handling\fP for details.

.br
classes are always derived from \fBVObject\fP.
.br
class \fBVObject\fP
is never reused directly.
.br
class \fBVObject\fP is abstract.
.br
class \fBVObject\fP contains 81 methods.
.PD 0
.TP 20
\fBowner of class:\fR
nobody.
.PD 0
.TP 20
\fBbaseclasses:\fR
.na
EvtHandler
.ad b
.PD 0
.TP 20
\fBsubclasses:\fR
.na
Clipper, CompositeVObject, Filler, GraphPath, GraphReference, ImageItem, LineItem, SplitBar, TextItem, Thumb, VObjectPair, View
.ad b
.PD 0
.TP 20
\fBflags:\fR
VObjFlags

.SH INSTANCE VARIABLES
.TP
.BR container " (private VObject *)"
is the \fBparent\fP of this \fBVObject\fP. VObjects are organized in a tree-like fashion and every VObject is \fB geometrically contained\fP in its parent. Therefore the container of a window (see \fIclass BlankWin\fP), which is the root of the \fBVObject\fP tree, is always \fBNULL\fP. When a window is opened the first time, the parent of a VObject initializes the \fBcontainer\fP of its inner \fBVObjects\fP through the \fImethod SetContainer\fP. The \fBcontainer\fP can be accessed through the \fImethod GetContainer\fP. The \fBcontainer\fP is also returned as next event handler by the \fImethod VObject::GetNextHandler\fP.
.br
See also \fItechnote VObject Tree\fP.

.TP
.BR contentRect " (public Rectangle)"
contains the bounding box, i.e. the \fBorigin and the extent\fP of the VObject. The origin is always relative to the next container of \fIclass Clipper\fP because every \fBClipper\fP maintains its own coordinate system.
The \fBcontentRect\fP is very often accessed directly, but it should be modified only via the appropriate methods.

.TP
.BR id " (private short)"
is a clear \fBidentification\fP of the VObject. It can be chosen either from predefined (\fBcId...\fP) or from user-defined constants which must be greater than \fIconst cIdFirstUser\fP. The default value of the \fBid\fP is \fIconst cIdNone\fP. The \fBid\fP is returned by the \fImethod GetId\fP and can be set with the \fImethod SetId\fP.
.br
The \fBid\fP need not be unique throughout the whole application. But it must identify a \fBVObject\fP in a given context. Dialogs examine the \fBid\fP in their \fBControl\fP method to find out which button has been pressed. Some dialog items pass the \fBid\fP to the \fImethod SendDown\fP to make other \fBVObjects\fP do certain things. E.g., \fImethod ChangeDialog::Control\fP, \fImethod Button::DoOnItem\fP or \fImethod OneOfCluster::SetItemOn\fP.


.SH INSTANCE METHOD LIST

.PD 0
.TP 20
accessing
ContentRect
.br
Enabled
.br
GetContainer
.br
GetExtent
.br
GetId
.br
GetOrigin
.br
Height
.br
IsOpen
.br
WantsKbdFocus
.br
Width

.PD 0
.TP 20
client interface
SetId

.PD 0
.TP 20
commands
GetMover
.br
GetStretcher

.PD 0
.TP 20
comparison
AsString

.PD 0
.TP 20
constructor
VObject

.PD 0
.TP 20
container handling
AddToClipper
.br
ContainerPoint
.br
FindContainerOfClass
.br
GetPortPoint
.br
GetView
.br
GetWindow
.br
Getclipper
.br
RemoveFromClipper
.br
SetContainer

.PD 0
.TP 20
content rectangle
Align
.br
ContainsPoint
.br
GetViewedRect
.br
Move
.br
SetContentRect
.br
SetExtent
.br
SetHeight
.br
SetOrigin
.br
SetWidth

.PD 0
.TP 20
detect
Detect
.br
FindItem
.br
FindItemPtr

.PD 0
.TP 20
drawing
Draw
.br
DrawAll
.br
DrawHighlight
.br
DrawInner
.br
Highlight
.br
Outline
.br
Outline2
.br
OutlineRect
.br
UpdateEvent

.PD 0
.TP 20
event handling
DispatchEvents
.br
DoCursorKeyCommand
.br
DoFunctionKeyCommand
.br
DoKeyCommand
.br
DoLeftButtonDownCommand
.br
DoMiddleButtonDownCommand
.br
DoOnItem
.br
DoOtherEventCommand
.br
DoRightButtonDownCommand
.br
GetCursor
.br
Input

.PD 0
.TP 20
flag handling
Close
.br
Disable
.br
Enable
.br
Open

.PD 0
.TP 20
focusing
Focus
.br
SetFocus

.PD 0
.TP 20
implementation
Init

.PD 0
.TP 20
invalidation
ForceRedraw
.br
InvalidateRect
.br
InvalidateViewRect

.PD 0
.TP 20
layout calculation
Base
.br
CalcExtent
.br
GetMinSize

.PD 0
.TP 20
mouse tracking
TrackInContent

.PD 0
.TP 20
miscellaneous
Compare
.br
GetNextHandler
.br
IsEqual
.br
PrintOn
.br
ReadFrom

.PD 0
.TP 20
printing
Print

.SH CATEGORIES

.na
Graphic Foundation
.ad b

.SH FILES
.PD 0
.TP 20
declaration:
\fBVObject.h

.SH HISTORY
joe	- Mon Apr 15 16:52 MET DST 1991 - first version
.br
joe	- Wed Jun 12 20:51 MET DST 1991 - reworked
.br
joe	- Thu Jun 13 23:30 MET DST 1991 - reworked after chris' review
.br
