.po -3
.TH "View::PasteData" n "automatically Mon Apr  8 20:17:22 1991" "ET++ version v2.2" "ET++ method description"
.SH NAME
View::PasteData \- instance method
.SH TEMPLATE
pasteCommand = Command *
PasteData(char * type, istream & inputStream)
.SH SPECIFIERS
public
abstract
virtual
.SH DESCRIPTION
\fBPasteData\fP is called when the \fBpaste\fP command is selected from the popup menu. But this menu item is only enabled if the \fImethod CanPaste\fP returns \fBTRUE\fP.
.br
\fBPasteData\fP must read the clipboard data from the \fBinputStream\fP and create an undoable \fBpasteCommand\fP to insert the read data into the data structure of \fBthis\fP. If nothing is read or the clipboard data cannot be pasted, then \fBgNoChanges\fP must be returned.
.br
For example, a subclass of \fBView\fP that expects objects of a certain class would implement \fBPasteData\fP like this:
.na
.nf

Command *MyView::PasteData (char *type, istream& is)
{
    if (strcmp(type, cDocTypeET) == 0)
    {
        Object *pastedObject = NULL;
        is \>\> pastedObject;
        if (pastedObject && pastedObject-\>IsKindOf(MyDesiredClass))
            return new MyPasteCommand (pastedObject);
    }
    return gNoChanges;
}

.ad b
.fi
A concrete example can be found in the \fImethod TextView::PasteData\fP.
.br
See also \fImethod SelectionToClipboard\fP.
Method is sometimes overridden.
.SH ARGUMENTS
.TP
\fBchar * type
.br
is the type of the clipboard data contained in the \fBinputStream\fP. Common types are \fBcDocTypeET\fP which indicates ET++ Object Input/Output format and \fBcDocTypeAscii\fP for pure ASCII text.
.TP
\fBistream & inputStream
.br
a stream created by the clipboard from which the clipboard data can be read
.SH RETURN ARGUMENT
.TP
\fBCommand * pasteCommand
.br
is either \fBgNoChanges\fP or an undoable command object which actually pastes the data into \fBthis\fP
.SH CATEGORIES

.na
clipboard, overrider interface
.ad b

.SH FIRST DEFINITION
class View
.SH FILES
.PD 0
.TP 20
implementation:
\fBView.C

