%{
#include <stdio.h>
#include "persim.h"
#include "funcs.h"

extern int linenum,inhelp,inamb,print_line;
extern char *program,*yacc_filename;
%}

%union
{
	int inum;
	double dnum;
	char *string;
}

%token AMBIGUOUS_KEYWORD EXIT_CMD HELP_CMD SHOW_CMD
%token FUNC_ATTR THRESHOLD_ATTR RUN_CMD VERBOSE_ATTR
%token <inum> DATA FILETYPE INT KEYWORD0 KEYWORD1 KEYWORD2 KEYWORD3
%token <inum> NODE_CMD KEYWORD4 NOTFILE
%token <dnum> FP
%token <string> STRING

%type <inum> help run show

%%
line:
		| line '\n'			{ wrapup_line(1); }
		| line command			{ wrapup_line(0); }
		;
command:	  error '\n' 			{ yyerrok; }
		| EXIT_CMD '\n'			{ YYACCEPT; }
		| ambiguous '\n'		{ yyerror(); }
		| help '\n'			{ help($1); }
		| KEYWORD0 '\n'			{ execute[$1](); }
		| KEYWORD1 '\n'			{ execute[$1](); }
		| KEYWORD2 '=' INT '\n'		{ execute[$1]($1,$3); }
		| KEYWORD2 '=' FP '\n'		{ execute[$1]($1,$3); }
		| KEYWORD3 DATA '"' STRING '"' '\n'
						{ execute[$1]($2,$4,UNKNOWN); }
		| KEYWORD3 DATA '"' STRING '"' ',' FILETYPE '\n'
						{ execute[$1]($2,$4,$7); }
		| KEYWORD3 DATA NOTFILE '\n' 	{ execute[$1]($2,NULL,$3); }
		| KEYWORD4 STRING FP FP '\n'	{ execute[$1]($2,$3,$4); }
		| NODE_CMD FUNC_ATTR STRING '\n'
						{ set_func($1,$3); }
		| NODE_CMD THRESHOLD_ATTR INT '\n'
						{ set_threshold($1,(double) $3); }
		| NODE_CMD THRESHOLD_ATTR FP '\n'
						{ set_threshold($1,$3); }
		| SHOW_CMD NODE_CMD '\n'	{ show(NODE,$2); }
		| run '\n'			{ run($1); }
		| show '\n'			{ show($1,0); }
		;
ambiguous:	  AMBIGUOUS_KEYWORD		{ }
		| AMBIGUOUS_KEYWORD '"' STRING '"'
						{ }
		| AMBIGUOUS_KEYWORD '=' INT 	{ }
		| AMBIGUOUS_KEYWORD '=' '"' STRING '"'
						{ }
		| AMBIGUOUS_KEYWORD '=' '"' STRING '"' ',' FILETYPE
						{ }
		| AMBIGUOUS_KEYWORD '=' NOTFILE
						{ }
		| AMBIGUOUS_KEYWORD FUNC_ATTR STRING
						{ }
		| AMBIGUOUS_KEYWORD THRESHOLD_ATTR INT
						{ }
		| AMBIGUOUS_KEYWORD THRESHOLD_ATTR FP
						{ }
		| AMBIGUOUS_KEYWORD VERBOSE_ATTR
						{ }
		| AMBIGUOUS_KEYWORD STRING FP FP
						{ }
		;
help:		  HELP_CMD 			{ $$ = -1; }
		| HELP_CMD EXIT_CMD		{ $$ = EXIT; }
		| HELP_CMD HELP_CMD		{ $$ = HELP; }
		| HELP_CMD KEYWORD0		{ $$ = $2; }
		| HELP_CMD KEYWORD1		{ $$ = $2; }
		| HELP_CMD KEYWORD2		{ $$ = $2; }
		| HELP_CMD KEYWORD3		{ $$ = $2; }
		| HELP_CMD KEYWORD4		{ $$ = $2; }
		| HELP_CMD NODE_CMD		{ $$ = NODE; }
		| HELP_CMD RUN_CMD		{ $$ = RUN; }
		| HELP_CMD SHOW_CMD		{ $$ = SHOW; }
		| HELP_CMD AMBIGUOUS_KEYWORD	{ $$ = AMBIGUOUS; }
		;
run:		  RUN_CMD			{ $$ = 0; }
		| RUN_CMD VERBOSE_ATTR		{ $$ = 1; }
		;
show:		  SHOW_CMD DATA			{ $$ = $2; }
		| SHOW_CMD KEYWORD1		{ $$ = $2; }
		| SHOW_CMD KEYWORD2		{ $$ = $2; }
		| SHOW_CMD KEYWORD3		{ $$ = $2; }
		| SHOW_CMD KEYWORD4		{ $$ = $2; }
		| SHOW_CMD AMBIGUOUS_KEYWORD	{ $$ = AMBIGUOUS; }
		;
%%  

/*This function is called when yacc has an error in parsing.

  Inputs:  none
  Outputs: none
  Locals:  none
  Globals: linenum       - current line number in file that yacc is reading
	   print_line    -
	   program       - name of program
	   stderr	 - standard error
	   NULL		 - 0
*/
yyerror()
{
	if(print_line)
	  if(inhelp) printf("\ninvalid command to ask for help");
	  else if(inamb)
		 printf("%s: ambiguous command on line %d\n",program,linenum);
	       else printf("\n%s: syntax error on line %d",program,linenum);
	else 
	  if(inhelp) puts("invalid command to ask for help");
	  else if(inamb) fputs("*** ambiguous command ***\n",stderr);
	       else fputs("*** syntax error ***\n",stderr);
}

wrapup_line(blank)
register int blank;
{
	++linenum;
	inhelp = 0;
	inamb = 0;

	prompt();
}

