%{
#include "grammar.h"
#include "persim.h"

int linenum = 1,inhelp = 0,inamb = 0,quotes = 0;
extern int print_line;
double atof();
char *strdup();
%}
COMMENT		(#.*\n)
MATCH_STRING	([a-zA-Z/][a-zA-Z0-9_\./]*)
NODE		(node[0-9]+)
NUMBER		([-+]?[0-9]+\.?[0-9]*)
%%
{COMMENT}	{ ++linenum; if(print_line) ECHO; prompt(); }
{NODE}		{ ECHO; yylval.inum = atoi(yytext+4); return(NODE_CMD); }
{MATCH_STRING}	{
		  int info;

		  if(print_line) ECHO;

		  if(quotes)
		    if((yylval.string = strdup(yytext)) == NULL) {
		      error();
		      exit(1);
		    } else return(STRING);

		  switch(yylval.inum = iskeyword(yytext,&info)) {
		    case EXIT:
			return(EXIT_CMD);
		    case HELP:
			inhelp = 1;
			return(HELP_CMD);
		    case NODE:
			yylval.inum = -1;
			return(NODE_CMD);
		    case RUN:
			return(RUN_CMD);
		    case SHOW:
			return(SHOW_CMD);
		    case UNKNOWN:
			if((yylval.inum = isqualifier(yytext,&info)) == UNKNOWN)
			  if((yylval.string = strdup(yytext)) == NULL) {
			    error();
			    exit(1);
			  } else return(STRING);
			else if(yylval.inum == AMBIGUOUS)
			       return(AMBIGUOUS_KEYWORD);
			     else return(info);
		    case AMBIGUOUS:
			inamb = 1;
			return(AMBIGUOUS_KEYWORD);
		    default:
			switch(info) {
			  case 0:	return(KEYWORD0);
			  case 1:	return(KEYWORD1);
			  case 2:	return(KEYWORD2);
			  case 3:	return(KEYWORD3);
			  case 4:	return(KEYWORD4);
			}
		  }
		}
{NUMBER}	{
		  if(print_line) ECHO;

		  if(index(yytext,'.')) {
		    yylval.dnum = atof(yytext);
		    return(FP);
		  } else {
		           yylval.inum = atoi(yytext);
		           return(INT);
			 }
		}
\?		{ if(print_line) putchar('?'); yylval.inum = HELP;
		  inhelp = 1; return(HELP_CMD); }
\"		{ if(print_line) ECHO; quotes ^= 1; return('\"'); }
\n		{ if(print_line) ECHO; return('\n'); }
[^ \t]		{ if(print_line) putchar(yytext[0]); return(yytext[0]); }
[= \t]          { if(print_line) putchar(yytext[0]); }
%%

/*This function is called when lex reaches EOF.  A value of 1 is returned
  to tell lex to finish up normally.

  Inputs:  none
  Outputs: 1 - tells lex to finish normally
  Locals:  none
  Globals: none
*/
int yywrap()
{
    return(1);
}

