#!/bin/sh
#
# UUHOST, Copyright 1992,   Jan-Piet Mens [Logix GmbH, Wiesbaden, Germany]
# License to freely use and distribute this software is hereby granted 
# by the author, subject to the condition that this copyright notice 
# remains intact.  The author retains the exclusive right to publish 
# derivative works based on this work, including, but not limited
# to, revised versions of this work.
#
# $Id: uuhost.S,v 1.1 1992/01/16 09:43:40 jpm Exp $
# $Log: uuhost.S,v $
# Revision 1.1  1992/01/16  09:43:40  jpm
# Initial revision
#
#
# Set this to the name of the directory containing the uucp maps (u.*)
MAPSDIR="__MAPSDIR__"
# This is the file where the UUHOST index lives
UUHOSTINDEX="__UUHOSTINDEX__"
#
# This is the program that splits up a `#N' line into individual machines
#
UUSPLITNAME="__BINDIR__/uusplit"
#
# CAT is either `cat' if maps are not compressed, or `zcat' if they *are*.
#
CAT="__CATPROG__"
#
# If you don't want to use my `uulookup', you'll have to work at this script.
#
LOOK="__LOOKPROG__"
PATHLOOKUP="__PATHLOOKUP__"
PAGER="__PAGEPROG__"

USAGE="$0 [-l] [-r region] [hostname]"

[ $# -eq 0 ] && { echo "$USAGE" >&2 ; exit 1 ; }
case "$1" in 
	-l)	ls -C ${MAPSDIR} | sed -e 's/\.Z//g' ; exit 0 ;;
	-r)	[ $# -eq 2 ] || { echo "$USAGE" >&2 ; exit 2 ; }
		REGION="${MAPSDIR}/$2"
		[ ! -r $REGION -a ! -r ${REGION}.Z ] && 
				{ echo "$0: region $2 non-existant"; exit ; }
		${CAT} $REGION | ${PAGER}
		exit 0 ;;
	-index)
		[ -w ${UUHOSTINDEX} ] || {
				echo "$UUHOSTINDEX is not writeable" >&2
				exit 2 ; }

		echo "Building index '${UUHOSTINDEX}'. Patience." >&2
		cd ${MAPSDIR}				# Go to dir
		ls u.* | while read mapname		# Find all u. files
		do
			echo ".\c" >&2
			${CAT} $mapname | 
			grep -n '^#N' | 		# Only `#N' entries
			${UUSPLITNAME} $mapname		# Build list
		done > ${UUHOSTINDEX}
		echo "\n" >&2
		#
		# there is a bug in my `sort'. Otherwise, I would have put
		# it in the pipeline above...
		#
		sort -f -u ${UUHOSTINDEX} -o ${UUHOSTINDEX}
		chmod 644 ${UUHOSTINDEX}
		exit 0 ;;
	-*)	echo "$USAGE" >&2 ; exit 2 ;;
esac

machine=$1

${LOOK} $machine ${UUHOSTINDEX} | 		# May give several per machine
	sort -1 +2 -u | 			# Give unique entries for mach
	while read mach map line		# Get Machinename, mapfile, line
	do
		${PATHLOOKUP} $mach		# E-mail address ?

		echo '\n'

		${CAT} ${MAPSDIR}/${map} |	# Output file...
		sed -n -e "$line,/^$/p"  |	# ... starting at `line'
		sed -e "s/^#N/#Name          /"	\
		    -e "s/^#S/#System-OS     /"	\
		    -e "s/^#O/#Organisation  /"	\
		    -e "s/^#F/#Forwarder(MX) /"	\
		    -e "s/^#R/#Remarks       /"	\
		    -e "s/^#E/#Electr.Adress /"	\
		    -e "s/^#C/#Contact       /"	\
		    -e "s/^#T/#Telephone     /"	\
		    -e "s/^#P/#Postal-Address/"	\
		    -e "s/^#L/#Lat.-Longitude/"	\
		    -e "s/^#W/#Written-By:   /"	\
		    -e "s/^#U/#Usenet links  /"
		echo "================== ${map} ======================="
	done | ${PAGER} 
