.po -3
.TH "Object::DoObserve" n "automatically Fri Mar 1 10:54:57 1991" "ET++ version v2.2" "ET++ method description"
.SH NAME
Object::DoObserve \- instance method
.SH TEMPLATE
void DoObserve(int objectId, int partCode, void * value, ObjPtr observedObject)
.SH SPECIFIERS
public
virtual
.SH DESCRIPTION
This method is called by the \fImethod Send\fP of the \fBobservedObject\fP to inform its observers about a change in its state. \fBDoObserve\fP is called for objects only which have previously been registered
as observers of the \fBobservedObject\fP through the \fImethod
AddObserver\fP.
.br
The first three arguments contain detailed information about the
change.
.br
If DoObserve has to be overridden because the inherited behaviour does not
handle some cases, \fBit is essential to partially override
DoObserve\fP. Not doing so would break the object communication patterns
already established. As a rule-of-thumb, DoObserve is best overridden in
post-order, i.e. handle the change message at the more specific level, and
then at the more general level.
.br
Clearly, the client may totally override it if he exploits the fact that
there is no need to call the super class' DoObserve. This is not considered
good programming practice since it introduces unnecessary remote
dependencies along an inheritance path. The minimal overhead introduced by
partially overriding DoOberve is neglectable in virtually all cases.
.br
Examples for some change messages can be found in the \fIclass Text\fP
and the \fIclass TextView\fP.
.br
See \fItechnote 'Change Propagation'\fP.
Method is sometimes overridden.
.SH ARGUMENTS
.TP
\fBint objectId
.br
either the id of the changed object or
\fIconst cIdNone\fP
.TP
\fBint partCode
.br
a code that describes the change message more detailed. It is usually one
of the \fBcPart...\fP constants.
.TP
\fBvoid * value
.br
an argument possibly nil whose type and semantics depend on the values of
\fBobjectId\fP and \fBpartCode\fP.
.TP
\fBObjPtr observedObject
.br
the observed object which has changed and called this method
.SH RETURN ARGUMENT
void

.SH CATEGORIES

.na
change propagation, overrider interface
.ad b

.SH FIRST DEFINITION
class Object
.SH FILES
.PD 0
.TP 20
implementation:
\fBObject.C

